#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.observed_holiday_base import ObservedHolidayBase, SUN_TO_NEXT_MON


class Cuba(ObservedHolidayBase, ChristianHolidays, InternationalHolidays):
    """Cuba holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Cuba>
        * [1984 (DEC 28)](https://web.archive.org/web/20240726190205/https://files.sld.cu/prevemi/files/2013/03/ley_49_codigo_trabajo_1984.pdf)
        * [2007 (NOV 19)](https://web.archive.org/web/20220705173115/https://www.gacetaoficial.gob.cu/sites/default/files/go_x_053_2007.pdf)
        * [2013 (DEC 20)](https://web.archive.org/web/20241125112840/http://www.gacetaoficial.gob.cu/es/ley-no-116-codigo-de-trabajo/)

    Certain holidays details:
        * Good Friday:
            * <https://web.archive.org/web/20250413192145/https://www.cnn.com/2012/03/31/world/americas/cuba-good-friday/index.html>
            * <https://web.archive.org/web/20241005041855/https://www.catholicnewsagency.com/news/29455/cuban-government-makes-good-friday-official-holiday>
        * Christmas Day:
            * <https://web.archive.org/web/20250413192307/https://time.com/vault/issue/1969-11-07/page/44/>
            * <https://web.archive.org/web/20250427131226/https://www.cnn.com/WORLD/9712/15/castro.christmas/>
            * <https://web.archive.org/web/20250413185514/https://www.independent.co.uk/news/cuba-ends-its-30year-ban-on-christmas-1193525.html>
            * <https://web.archive.org/web/20220508094546/https://www.ilo.org/dyn/travail/docs/1320/DECRETO-LEY%20No.%20189-1998.pdf>

    For holidays that can be moved to a Monday if they fall on a Sunday, between 1984
    and 2013, the State Committee of Work and Social Security would determine if they
    would be moved to the Monday, or if they would stay on the Sunday, presumably
    depending on quotas. After 2013, they always move to Monday. I could not find any
    records of this, so I implemented this making it always go to the next Monday.
    """

    country = "CU"
    default_language = "es"
    # %s (observed).
    observed_label = tr("%s (observado)")
    supported_languages = ("en_US", "es", "uk")
    # This calendar only works from 1959 onwards.
    start_year = 1959

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        kwargs.setdefault("observed_rule", SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # Liberation Day.
        jan_1 = self._add_holiday_jan_1(tr("Triunfo de la Revolución"))
        if self._year <= 2013:
            self._add_observed(jan_1)

        if self._year >= 2008:
            #  Victory Day.
            self._add_holiday_jan_2(tr("Día de la Victoria"))

        # Granted temporarily in 2012 and 2013.
        # Permanently granted in 2013 decree for 2014 and onwards.
        if self._year >= 2012:
            # Good Friday.
            self._add_good_friday(tr("Viernes Santo"))

        # International Workers' Day.
        self._add_observed(self._add_labor_day(tr("Día Internacional de los Trabajadores")))

        # Commemoration of the Assault of the Moncada garrison.
        name = tr("Conmemoración del asalto a Moncada")
        self._add_holiday_jul_25(name)
        self._add_holiday_jul_27(name)

        # Day of the National Rebellion.
        self._add_holiday_jul_26(tr("Día de la Rebeldía Nacional"))

        # Independence Day.
        self._add_observed(self._add_holiday_oct_10(tr("Inicio de las Guerras de Independencia")))

        # In 1969, Christmas was cancelled for the sugar harvest but then was cancelled for good.
        # In 1997, Christmas was temporarily back for the Pope's visit.
        # In 1998, Christmas returned for good.
        if self._year <= 1968 or self._year >= 1997:
            # Christmas Day.
            self._add_christmas_day(tr("Día de Navidad"))

        if self._year >= 2007:
            # New Year's Eve.
            self._add_new_years_eve(tr("Fiesta de Fin de Año"))


class CU(Cuba):
    pass


class CUB(Cuba):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from collections.abc import Iterable
from datetime import date

from holidays.calendars.gregorian import _timedelta


class EasternCalendarHolidays:
    """
    Eastern calendar holidays base class.
    """

    def _add_eastern_calendar_holiday(
        self,
        name: str,
        dt_estimated: tuple[date | None, bool],
        *,
        show_estimated: bool = True,
        days_delta: int = 0,
    ) -> date | None:
        """
        Add Eastern (Buddhist, Chinese, Hindu, Islamic, Mongolian) calendar holiday.

        Adds customizable estimation label to holiday name if holiday date is an estimation.
        """
        dt, is_estimated = dt_estimated
        if days_delta and dt:
            dt = _timedelta(dt, days_delta)

        return (
            self._add_holiday(
                self.tr(self.estimated_label) % self.tr(name)
                if is_estimated and show_estimated
                else name,
                dt,
            )
            if dt
            else None
        )

    def _add_eastern_calendar_holiday_set(
        self,
        name: str,
        dts_estimated: Iterable[tuple[date, bool]],
        *,
        show_estimated: bool = True,
        days_delta: int = 0,
    ) -> set[date]:
        """
        Add Eastern (Buddhist, Chinese, Hindu, Islamic, Mongolian) calendar holidays.

        Adds customizable estimation label to holiday name if holiday date is an estimation.
        """

        return {
            dt
            for dt_estimated in dts_estimated
            if (
                dt := self._add_eastern_calendar_holiday(
                    name, dt_estimated, show_estimated=show_estimated, days_delta=days_delta
                )
            )
        }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.constants import OPTIONAL, PUBLIC
from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.holiday_base import HolidayBase


class Portugal(HolidayBase, ChristianHolidays, InternationalHolidays):
    """Portugal holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Portugal>
        * [Labour Day](https://web.archive.org/web/20240502184140/https://www.e-konomista.pt/dia-do-trabalhador/)
        * Portugal Day - Decreto 17.171
        * Restoration of Independence Day - Gazeta de Lisboa, 8 de Dezembro
            de 1823 (n.º 290), pp. 1789 e 1790
        * Azores:
            * <https://web.archive.org/web/20240811191206/https://files.dre.pt/1s/1980/08/19200/23052305.pdf>
        * Madeira:
            * <https://web.archive.org/web/20230729205620/https://files.dre.pt/1s/1979/11/25900/28782878.pdf>
            * <https://web.archive.org/web/20230730094358/https://files.dre.pt/1s/1989/02/02800/04360436.pdf>
            * <https://web.archive.org/web/20230729210821/https://files.dre.pt/1s/2002/11/258a00/71837183.pdf>
    """

    country = "PT"
    default_language = "pt_PT"
    supported_categories = (OPTIONAL, PUBLIC)
    start_year = 1801
    subdivisions = (
        "01",  # Aveiro.
        "02",  # Beja.
        "03",  # Braga.
        "04",  # Bragança.
        "05",  # Castelo Branco.
        "06",  # Coimbra.
        "07",  # Évora.
        "08",  # Faro.
        "09",  # Guarda.
        "10",  # Leiria.
        "11",  # Lisboa.
        "12",  # Portalegre.
        "13",  # Porto.
        "14",  # Santarém.
        "15",  # Setúbal.
        "16",  # Viana do Castelo.
        "17",  # Vila Real.
        "18",  # Viseu.
        "20",  # Região Autónoma dos Açores.
        "30",  # Região Autónoma da Madeira.
    )
    subdivisions_aliases = {
        "Aveiro": "01",
        "Beja": "02",
        "Braga": "03",
        "Bragança": "04",
        "Castelo Branco": "05",
        "Coimbra": "06",
        "Évora": "07",
        "Faro": "08",
        "Guarda": "09",
        "Leiria": "10",
        "Lisboa": "11",
        "Portalegre": "12",
        "Porto": "13",
        "Santarém": "14",
        "Setúbal": "15",
        "Viana do Castelo": "16",
        "Vila Real": "17",
        "Viseu": "18",
        "Região Autónoma dos Açores": "20",
        "Região Autónoma da Madeira": "30",
    }
    supported_languages = ("en_US", "pt_PT", "uk")
    _deprecated_subdivisions = ("Ext",)

    def __init__(self, *args, **kwargs) -> None:
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("Ano Novo"))

        # Carnival is no longer a holiday, but some companies let workers off.
        # TODO: recollect the years in which it was a public holiday.

        # Good Friday.
        self._add_good_friday(tr("Sexta-feira Santa"))

        # Easter Sunday.
        self._add_easter_sunday(tr("Páscoa"))

        # Revoked holidays in 2013-2015.
        if self._year <= 2012 or self._year >= 2016:
            # Corpus Christi.
            self._add_corpus_christi_day(tr("Corpo de Deus"))

            if self._year >= 1910:
                # Republic Day.
                self._add_holiday_oct_5(tr("Implantação da República"))

            # All Saints' Day.
            self._add_all_saints_day(tr("Dia de Todos os Santos"))

            if self._year >= 1823:
                # Restoration of Independence Day.
                self._add_holiday_dec_1(tr("Restauração da Independência"))

        if self._year >= 1974:
            # Freedom Day.
            self._add_holiday_apr_25(tr("Dia da Liberdade"))

            # Labor Day.
            self._add_labor_day(tr("Dia do Trabalhador"))

        if self._year >= 1911:
            if 1933 <= self._year <= 1973:
                # Day of Camões, Portugal, and the Portuguese Race.
                name = tr("Dia de Camões, de Portugal e da Raça")
            elif self._year >= 1978:
                # Day of Portugal, Camões, and the Portuguese Communities.
                name = tr("Dia de Portugal, de Camões e das Comunidades Portuguesas")
            else:
                # Portugal Day.
                name = tr("Dia de Portugal")
            self._add_holiday_jun_10(name)

        # Assumption Day.
        self._add_assumption_of_mary_day(tr("Assunção de Nossa Senhora"))

        # Immaculate Conception.
        self._add_immaculate_conception_day(tr("Imaculada Conceição"))

        # Christmas Day.
        self._add_christmas_day(tr("Dia de Natal"))

    def _populate_optional_holidays(self):
        """
        Adds extended days that most people have as a bonus from their
        companies:

        - Carnival
        - the day before and after xmas
        - the day before the new year
        - Lisbon's city holiday
        """

        # TODO: add bridging days:
        # - get Holidays that occur on Tuesday  and add Monday (-1 day)
        # - get Holidays that occur on Thursday and add Friday (+1 day)

        # Carnival.
        self._add_carnival_tuesday(tr("Carnaval"))

        # Saint Anthony's Day.
        self._add_saint_anthonys_day(tr("Dia de Santo António"))

        # Christmas Eve.
        self._add_christmas_eve(tr("Véspera de Natal"))

        # Second Day of Christmas.
        self._add_christmas_day_two(tr("26 de Dezembro"))

        # New Year's Eve.
        self._add_new_years_eve(tr("Véspera de Ano Novo"))

    def _populate_subdiv_holidays(self):
        if self._year >= 1911:
            super()._populate_subdiv_holidays()

        if self.subdiv == "Ext":
            self._populate_optional_holidays()

    def _populate_subdiv_01_public_holidays(self):
        # Saint Joanna's Day.
        self._add_holiday_may_12(tr("Dia de Santa Joana"))

    def _populate_subdiv_02_public_holidays(self):
        # Ascension Day.
        self._add_ascension_thursday(tr("Quinta-feira da Ascensão"))

    def _populate_subdiv_03_public_holidays(self):
        # Saint John's Day.
        self._add_saint_johns_day(tr("Dia de São João"))

    def _populate_subdiv_04_public_holidays(self):
        # Feast of Our Lady of Graces.
        self._add_holiday_aug_22(tr("Dia de Nossa Senhora das Graças"))

    def _populate_subdiv_05_public_holidays(self):
        # Feast of Our Lady of Mércoles.
        self._add_holiday_16_days_past_easter(tr("Dia de Nossa Senhora de Mércoles"))

    def _populate_subdiv_06_public_holidays(self):
        # Saint Elizabeth's Day.
        self._add_holiday_jul_4(tr("Dia de Santa Isabel"))

    def _populate_subdiv_07_public_holidays(self):
        # Saint Peter's Day.
        self._add_saints_peter_and_paul_day(tr("Dia de São Pedro"))

    def _populate_subdiv_08_public_holidays(self):
        # Municipal Holiday of Faro.
        self._add_holiday_sep_7(tr("Dia do Município de Faro"))

    def _populate_subdiv_09_public_holidays(self):
        # Municipal Holiday of Guarda.
        self._add_holiday_nov_27(tr("Dia do Município da Guarda"))

    def _populate_subdiv_10_public_holidays(self):
        # Municipal Holiday of Leiria.
        self._add_holiday_may_22(tr("Dia do Município de Leiria"))

    def _populate_subdiv_11_public_holidays(self):
        # Saint Anthony's Day.
        self._add_saint_anthonys_day(tr("Dia de Santo António"))

    def _populate_subdiv_12_public_holidays(self):
        # Municipal Holiday of Portalegre.
        self._add_holiday_may_23(tr("Dia do Município de Portalegre"))

    def _populate_subdiv_13_public_holidays(self):
        # Saint John's Day.
        self._add_saint_johns_day(tr("Dia de São João"))

    def _populate_subdiv_14_public_holidays(self):
        # Saint Joseph's Day.
        self._add_saint_josephs_day(tr("Dia de São José"))

    def _populate_subdiv_15_public_holidays(self):
        # Bocage Day.
        self._add_holiday_sep_15(tr("Dia de Bocage"))

    def _populate_subdiv_16_public_holidays(self):
        # Feast of Our Lady of Sorrows.
        self._add_holiday_aug_20(tr("Dia de Nossa Senhora da Agonia"))

    def _populate_subdiv_17_public_holidays(self):
        # Saint Anthony's Day.
        self._add_saint_anthonys_day(tr("Dia de Santo António"))

    def _populate_subdiv_18_public_holidays(self):
        # Saint Matthew's Day.
        self._add_holiday_sep_21(tr("Dia de São Mateus"))

    def _populate_subdiv_20_public_holidays(self):
        if self._year >= 1981:
            # Day of the Autonomous Region of the Azores.
            self._add_whit_monday(tr("Dia da Região Autónoma dos Açores"))

    def _populate_subdiv_30_public_holidays(self):
        if self._year >= 1979:
            self._add_holiday_jul_1(
                # Day of the Autonomous Region of Madeira and the Madeiran Communities.
                tr("Dia da Região Autónoma da Madeira e das Comunidades Madeirenses")
                if self._year >= 1989
                # Day of the Autonomous Region of Madeira.
                else tr("Dia da Região Autónoma da Madeira")
            )

        if self._year >= 2002:
            # 1st Octave.
            self._add_christmas_day_two(tr("Primeira Oitava"))


class PT(Portugal):
    pass


class PRT(Portugal):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import FEB, OCT
from holidays.groups import (
    ChristianHolidays,
    InternationalHolidays,
    IslamicHolidays,
    StaticHolidays,
)
from holidays.observed_holiday_base import ObservedHolidayBase, SUN_TO_NEXT_MON


class IvoryCoast(
    ObservedHolidayBase, ChristianHolidays, InternationalHolidays, IslamicHolidays, StaticHolidays
):
    """Ivory Coast holidays.

    References:
        * [Decree No. 96-205](https://web.archive.org/web/20240701120937/http://www.droit-afrique.com/uploads/RCI-Decret-1996-205-jours-feries.pdf)
        * Decree No. 2011-371:
            * [page 1](https://web.archive.org/web/20180617165811/http://www.cgeci.org/cgeci/docs/documents/Doc-ferie-1.pdf)
            * [page 2](https://web.archive.org/web/20180826205106/http://www.cgeci.org/cgeci/docs/documents/Doc-ferie-2.pdf)
        * <https://fr.wikipedia.org/wiki/Fêtes_et_jours_fériés_en_Côte_d'Ivoire>
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Ivory_Coast>
        * <https://web.archive.org/web/20250408210549/https://www.timeanddate.com/holidays/ivory-coast/>
        * [National Peace Day](https://en.wikipedia.org/wiki/Ivory_Coast#Independence)

    Note:
        The oldest decree available online that underpins the public holidays defined here
        for the Ivory Coast is Decree no. 96-205 of March 7, 1996.

        In Islamic calendar, days begin at sunset. The naming convention "day after" refers
        to the daylight hours following the night of the celebration, which is technically
        the same Gregorian calendar day.

        According to Decree no. 2011-371 of 4 November 2011, if Eid al-Fitr, Eid al-Adha
        or Christmas Day falls on a Sunday, the following Monday is also a holiday.
    """

    country = "CI"
    default_language = "fr"
    # %s (estimated).
    estimated_label = tr("%s (estimé)")
    # Day after the %s.
    observed_label = tr("Lendemain de la %s")
    # Day after the %s (estimated).
    observed_estimated_label = tr("Lendemain de la %s (estimé)")
    start_year = 1997
    supported_languages = ("en_CI", "en_US", "fr")

    def __init__(self, *args, islamic_show_estimated: bool = False, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        ChristianHolidays.__init__(self)
        # The observed dates for the Ivory Coast's islamic holidays have been verified against
        # local references (COSIM) and align with the default Umm al-Qura calculations.
        IslamicHolidays.__init__(self, show_estimated=islamic_show_estimated)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, IvoryCoastStaticHolidays)
        kwargs.setdefault("observed_rule", SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("1er janvier"))

        # Easter Monday.
        self._add_easter_monday(tr("Lundi de Pâques"))

        # Labor Day.
        self._add_observed(self._add_labor_day(tr("Fête du travail")))

        # Ascension Day.
        self._add_ascension_thursday(tr("Jour de l'Ascension"))

        # Whit Monday.
        self._add_whit_monday(tr("Lundi de Pentecôte"))

        # Independence Day.
        self._add_observed(self._add_holiday_aug_7(tr("Fête Nationale")))

        # Assumption Day.
        self._add_assumption_of_mary_day(tr("Fête de l'Assomption"))

        # National Peace Day.
        self._add_holiday_nov_15(tr("Journée Nationale de la Paix"))

        # All Saints' Day.
        self._add_all_saints_day(tr("Fête de la Toussaint"))

        if self._year <= 2000:
            self._add_holiday_dec_7(
                # Anniversary of death of President Felix Houphouet-Boigny.
                tr("Anniversaire du décès du Président Felix Houphouet-Boigny")
            )

        # Christmas Day.
        dt = self._add_christmas_day(tr("Fête de Noël"))
        if self._year >= 2011:
            self._add_observed(dt)

        # Day after Prophet's Birthday.
        self._add_mawlid_day(tr("Lendemain de l'Anniversaire de la Naissance du Prophète Mahomet"))

        # Day after Night of Power.
        self._add_laylat_al_qadr_day(tr("Lendemain de la Nuit du Destin"))

        # Eid al-Fitr.
        for dt in self._add_eid_al_fitr_day(tr("Fête de fin du Ramadan")):
            if self._year >= 2012:
                self._add_observed(dt)

        # Eid al-Adha.
        for dt in self._add_eid_al_adha_day(tr("Fête de la Tabaski")):
            if self._year >= 2012:
                self._add_observed(dt)


class CI(IvoryCoast):
    pass


class CIV(IvoryCoast):
    pass


class IvoryCoastStaticHolidays:
    """Ivory Coast special holidays.

    References:
        * [2010 Presidential Election](https://web.archive.org/web/20250427185110/https://www.gouv.ci/_actualite-article.php?d=4.&recordID=1255&p=366)
        * [2024 AFCON](https://web.archive.org/web/20250429075000/https://apanews.net/public-holiday-as-cote-divoire-wins-afcon-trophy/)
    """

    special_public_holidays = {
        # Public holiday for Presidential election preparation.
        2010: (OCT, 29, tr("Jour férié pour la préparation de l'élection présidentielle")),
        # 2024 African Cup of Nations Victory.
        2024: (FEB, 12, tr("Victoire à la Coupe d'Afrique des Nations 2024")),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.constants import BANK, PROTESTANT, PUBLIC
from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.holiday_base import HolidayBase


class Austria(HolidayBase, ChristianHolidays, InternationalHolidays):
    """Austria holidays.

    !!! note "Note"
        Although sources for RGBl. Nr. 21, RGBl. Nr. 125, StGBl. Nr. 282, BGBl. Nr. 421,
        BGBl. Nr. 548, and RGBl. I S. 790 are readily available, they either do not specify
        any public holiday observances or are limited to very specific professions.

    References:
        * <https://de.wikipedia.org/wiki/Feiertage_in_Österreich>
        * [BGBl. Nr. 31/1933](http://archive.today/2026.04.08-130337/https://alex.onb.ac.at/cgi-content/alex?apm=0&aid=bgb&datum=19330004&seite=00000103&size=45)
        * [StGBl. Nr. 116/1945](https://web.archive.org/web/20260408120714/https://www.ris.bka.gv.at/Dokumente/BgblPdf/1945_116_0/1945_116_0.pdf)
        * [BGBl. Nr. 173/1949](https://web.archive.org/web/20260408143732/https://ris.bka.gv.at/Dokumente/BgblPdf/1949_173_0/1949_173_0.pdf)
        * [BGBl. Nr. 227/1955 and BGBl. Nr. 228/1955](https://web.archive.org/web/20260408143052/https://ris.bka.gv.at/Dokumente/BgblPdf/1955_227_0/1955_227_0.pdf)
        * [BGBl. Nr. 153/1957](https://web.archive.org/web/20250521041322/https://www.ris.bka.gv.at/Dokumente/BgblPdf/1957_153_0/1957_153_0.pdf)
        * [BGBl. Nr. 264/1967](https://web.archive.org/web/20241220074402/https://ris.bka.gv.at/Dokumente/BgblPdf/1967_264_0/1967_264_0.pdf)
        * [BGBl. Nr. 144/1983](https://web.archive.org/web/20260408111100/https://www.ris.bka.gv.at/Dokumente/BgblPdf/1983_144_0/1983_144_0.pdf)
        * [BGBl. I Nr. 191/1999](https://web.archive.org/web/20260330075009/https://www.ris.bka.gv.at/Dokumente/BgblPdf/1999_191_1/1999_191_1.pdf)
        * [BGBl. I Nr. 113/2006](https://web.archive.org/web/20260408114618/https://www.ris.bka.gv.at/Dokumente/BgblAuth/BGBLA_2006_I_113/BGBLA_2006_I_113.pdf)
        * [BGBl. I Nr. 22/2019](https://web.archive.org/web/20260408111719/https://www.ris.bka.gv.at/Dokumente/BgblAuth/BGBLA_2019_I_22/BGBLA_2019_I_22.pdf)
        * <https://web.archive.org/web/20260408155524/https://www.nachrichten.at/archivierte-artikel/serien/wir-oberoesterreicher/Vom-Maertyrer-zum-Patron;art11547,68941>
    """

    country = "AT"
    default_language = "de"
    # BGBl. Nr. 31/1933 became in effect on June 1st, 1933.
    start_year = 1934
    subdivisions = (
        "1",  # Burgenland.
        "2",  # Kärnten.
        "3",  # Niederösterreich.
        "4",  # Oberösterreich.
        "5",  # Salzburg.
        "6",  # Steiermark.
        "7",  # Tirol.
        "8",  # Vorarlberg.
        "9",  # Wien.
    )
    subdivisions_aliases = {
        "Burgenland": "1",
        "Bgld": "1",
        "B": "1",
        "Kärnten": "2",
        "Ktn": "2",
        "K": "2",
        "Niederösterreich": "3",
        "NÖ": "3",
        "N": "3",
        "Oberösterreich": "4",
        "OÖ": "4",
        "O": "4",
        "Salzburg": "5",
        "Sbg": "5",
        "S": "5",
        "Steiermark": "6",
        "Stmk": "6",
        "St": "6",
        "Tirol": "7",
        "T": "7",
        "Vorarlberg": "8",
        "Vbg": "8",
        "V": "8",
        "Wien": "9",
        "W": "9",
    }
    supported_categories = (BANK, PROTESTANT, PUBLIC)
    supported_languages = ("de", "en_US", "th", "uk")

    def __init__(self, *args, **kwargs) -> None:
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("Neujahr"))

        # Removed via StGBl. Nr. 116/1945.
        # Re-added via BGBl. Nr. 173/1949.
        if self._year <= 1945 or self._year >= 1950:
            # Epiphany.
            self._add_epiphany_day(tr("Heilige Drei Könige"))

        # Easter Monday.
        self._add_easter_monday(tr("Ostermontag"))

        # Added via via StGBl. Nr. 116/1945.
        if self._year >= 1946:
            # Labor Day.
            self._add_labor_day(tr("Staatsfeiertag"))

        # Ascension Day.
        self._add_ascension_thursday(tr("Christi Himmelfahrt"))

        # Whit Monday.
        self._add_whit_monday(tr("Pfingstmontag"))

        # Added via BGBl. Nr. 31/1933.
        # Removed via StGBl. Nr. 116/1945.
        if self._year <= 1945:
            # Saint Peter and Saint Paul's Day.
            self._add_saints_peter_and_paul_day(tr("Peter und Paul"))

        # Corpus Christi.
        self._add_corpus_christi_day(tr("Fronleichnam"))

        # Assumption Day.
        self._add_assumption_of_mary_day(tr("Mariä Himmelfahrt"))

        # Added via BGBl. Nr. 264/1967.
        if self._year >= 1967:
            # National Day.
            self._add_holiday_oct_26(tr("Nationalfeiertag"))

        # All Saints' Day.
        self._add_all_saints_day(tr("Allerheiligen"))

        # Removed via StGBl. Nr. 116/1945.
        # Re-added via BGBl. Nr. 227/1955.
        if self._year <= 1944 or self._year >= 1955:
            # Immaculate Conception.
            self._add_immaculate_conception_day(tr("Mariä Empfängnis"))

        # Christmas Day.
        self._add_christmas_day(tr("Christtag"))

        # Saint Stephen's Day.
        self._add_christmas_day_two(tr("Stephanstag"))

    def _populate_bank_holidays(self):
        # Good Friday.
        self._add_good_friday(tr("Karfreitag"))

        # Christmas Eve.
        self._add_christmas_eve(tr("Heiliger Abend"))

        # New Year's Eve.
        self._add_new_years_eve(tr("Silvester"))

    def _populate_protestant_holidays(self):
        # Added via BGBl. Nr. 228/1955.
        # Removed via BGBl. I Nr. 22/2019.
        if 1956 <= self._year <= 2018:
            # Good Friday.
            self._add_good_friday(tr("Karfreitag"))

    def _populate_subdiv_1_bank_holidays(self):
        # Saint Martin's Day.
        self._add_saint_martins_day(tr("Hl. Martin"))

    def _populate_subdiv_2_bank_holidays(self):
        # Saint Joseph's Day.
        self._add_saint_josephs_day(tr("Hl. Josef"))

        # 1920 Carinthian plebiscite.
        self._add_holiday_oct_10(tr("Tag der Volksabstimmung"))

    def _populate_subdiv_3_bank_holidays(self):
        # Saint Leopold's Day.
        self._add_holiday_nov_15(tr("Hl. Leopold"))

    def _populate_subdiv_4_bank_holidays(self):
        # Appointed as Saint of Upper Austrian via Provincial Government decision
        # on March 17th, 2004.
        if self._year >= 2004:
            # Saint Florian's Day.
            self._add_holiday_may_4(tr("Hl. Florian"))

    def _populate_subdiv_5_bank_holidays(self):
        # Saint Rupert's Day.
        self._add_holiday_sep_24(tr("Hl. Rupert"))

    def _populate_subdiv_6_bank_holidays(self):
        # Saint Joseph's Day.
        self._add_saint_josephs_day(tr("Hl. Josef"))

    def _populate_subdiv_7_bank_holidays(self):
        # Saint Joseph's Day.
        self._add_saint_josephs_day(tr("Hl. Josef"))

    def _populate_subdiv_8_bank_holidays(self):
        # Saint Joseph's Day.
        self._add_saint_josephs_day(tr("Hl. Josef"))

    def _populate_subdiv_9_bank_holidays(self):
        # Saint Leopold's Day.
        self._add_holiday_nov_15(tr("Hl. Leopold"))


class AT(Austria):
    pass


class AUT(Austria):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.holiday_base import HolidayBase


class ElSalvador(HolidayBase, ChristianHolidays, InternationalHolidays):
    """El Salvador holidays.

    References:
        * [Labor Code 1972](https://web.archive.org/web/20240918054537/http://www.transparencia.gob.sv/institutions/gd-usulutan/documents/192280/download)
        * <https://web.archive.org/web/20250308121509/https://www.timeanddate.com/holidays/el-salvador/>
        * <https://web.archive.org/web/20250122083353/https://www.officeholidays.com/countries/el-salvador>
    """

    country = "SV"
    default_language = "es"
    # Labor Code 1972.
    start_year = 1973
    subdivisions = (
        "AH",  # Ahuachapán.
        "CA",  # Cabañas.
        "CH",  # Chalatenango.
        "CU",  # Cuscatlán.
        "LI",  # La Libertad.
        "MO",  # Morazán.
        "PA",  # La Paz.
        "SA",  # Santa Ana.
        "SM",  # San Miguel.
        "SO",  # Sonsonate.
        "SS",  # San Salvador.
        "SV",  # San Vicente.
        "UN",  # La Unión.
        "US",  # Usulután.
    )
    subdivisions_aliases = {
        "Ahuachapán": "AH",
        "Cabañas": "CA",
        "Chalatenango": "CH",
        "Cuscatlán": "CU",
        "La Libertad": "LI",
        "Morazán": "MO",
        "La Paz": "PA",
        "Santa Ana": "SA",
        "San Miguel": "SM",
        "Sonsonate": "SO",
        "San Salvador": "SS",
        "San Vicente": "SV",
        "La Unión": "UN",
        "Usulután": "US",
    }
    supported_languages = ("en_US", "es", "uk")

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("Año Nuevo"))

        # Maundy Thursday.
        self._add_holy_thursday(tr("Jueves Santo"))

        # Good Friday.
        self._add_good_friday(tr("Viernes Santo"))

        # Holy Saturday.
        self._add_holy_saturday(tr("Sábado Santo"))

        # Labor Day.
        self._add_labor_day(tr("Día del Trabajo"))

        # Legislative Decree #399 from Apr 14, 2016.
        if self._year >= 2016:
            # Mother's Day.
            self._add_holiday_may_10(tr("Día de la Madre"))

        # Legislative Decree #208 from Jun 17, 2012.
        if self._year >= 2013:
            # Father's Day.
            self._add_holiday_jun_17(tr("Día del Padre"))

        # Celebrations of San Salvador.
        self._add_holiday_aug_6(tr("Celebración del Divino Salvador del Mundo"))

        # Independence Day.
        self._add_holiday_sep_15(tr("Día de la Independencia"))

        # All Souls' Day.
        self._add_all_souls_day(tr("Día de los Difuntos"))

        # Christmas Day.
        self._add_christmas_day(tr("Navidad"))

    def _populate_subdiv_ss_public_holidays(self):
        # Feast of San Salvador.
        name = tr("Fiesta de San Salvador")
        self._add_holiday_aug_3(name)
        self._add_holiday_aug_5(name)


class SV(ElSalvador):
    pass


class SLV(ElSalvador):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.observed_holiday_base import ObservedHolidayBase, SUN_TO_NEXT_MON, SUN_TO_NEXT_TUE


class SaintLucia(ObservedHolidayBase, ChristianHolidays, InternationalHolidays):
    """Saint Lucia holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Saint_Lucia>
        * <https://web.archive.org/web/20250415223705/https://www.timeanddate.com/holidays/saint-lucia/>
        * <https://web.archive.org/web/20241201164546/https://archive.stlucia.gov.lc/saint_lucia/public_holidays.htm>
        * <https://web.archive.org/web/20220227084500/https://archive.stlucia.gov.lc/stluciasilver/national_holidays.htm>
        * <https://web.archive.org/web/20160314100648/http://www.stluciachamber.org/uploadedImages/contentImg/file/List%20of%20Holidays%20for%202015%20(1).pdf>
    """

    country = "LC"
    default_language = "en_LC"
    # %s (observed).
    observed_label = tr("%s (observed)")
    supported_languages = ("en_LC", "en_US")
    start_year = 1979

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        kwargs.setdefault("observed_rule", SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_observed(self._add_new_years_day(tr("New Year's Day")), rule=SUN_TO_NEXT_TUE)

        # New Year's Holiday.
        self._add_observed(self._add_new_years_day_two(tr("New Year's Holiday")))

        # Independence Day.
        self._add_observed(self._add_holiday_feb_22(tr("Independence Day")))

        # Good Friday.
        self._add_good_friday(tr("Good Friday"))

        # Easter Monday.
        self._add_easter_monday(tr("Easter Monday"))

        # Labor Day.
        self._add_observed(self._add_labor_day(tr("Labour Day")))

        # Whit Monday.
        self._add_whit_monday(tr("Whit Monday"))

        # Corpus Christi.
        self._add_corpus_christi_day(tr("Corpus Christi"))

        # Emancipation Day.
        self._add_observed(self._add_holiday_aug_1(tr("Emancipation Day")))

        # Thanksgiving Day.
        self._add_holiday_1st_mon_of_oct(tr("Thanksgiving Day"))

        # National Day.
        self._add_observed(self._add_holiday_dec_13(tr("National Day")))

        # Christmas Day.
        self._add_observed(self._add_christmas_day(tr("Christmas Day")), rule=SUN_TO_NEXT_TUE)

        # Boxing Day.
        self._add_observed(self._add_christmas_day_two(tr("Boxing Day")))


class LC(SaintLucia):
    pass


class LCA(SaintLucia):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from holidays.countries.france import France
from holidays.mixins.child_entity import ChildEntity


class Mayotte(ChildEntity, France):
    """Mayotte holidays.

    Alias of a French subdivision that is also officially assigned
    its own country code in ISO 3166-1.

    References:
        * <https://en.wikipedia.org/wiki/Mayotte>
        * <https://en.wikipedia.org/wiki/Public_holidays_in_France>
    """

    country = "YT"
    parent_entity = France
    parent_entity_subdivision_code = "976"
    # Sold to France on April 25th, 1841.
    start_year = 1842


class HolidaysYT(Mayotte):
    pass


class YT(Mayotte):
    pass


class MYT(Mayotte):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from holidays.countries.afghanistan import Afghanistan, AF, AFG
from holidays.countries.aland_islands import AlandIslands, AX, ALA, HolidaysAX
from holidays.countries.albania import Albania, AL, ALB
from holidays.countries.algeria import Algeria, DZ, DZA
from holidays.countries.american_samoa import AmericanSamoa, AS, ASM, HolidaysAS
from holidays.countries.andorra import Andorra, AD, AND
from holidays.countries.angola import Angola, AO, AGO
from holidays.countries.anguilla import Anguilla, AI, AIA
from holidays.countries.antarctica import Antarctica, AQ, ATA
from holidays.countries.antigua_and_barbuda import AntiguaAndBarbuda, AG, ATG
from holidays.countries.argentina import Argentina, AR, ARG
from holidays.countries.armenia import Armenia, AM, ARM
from holidays.countries.aruba import Aruba, AW, ABW
from holidays.countries.australia import Australia, AU, AUS
from holidays.countries.austria import Austria, AT, AUT
from holidays.countries.azerbaijan import Azerbaijan, AZ, AZE
from holidays.countries.bahamas import Bahamas, BS, BHS
from holidays.countries.bahrain import Bahrain, BH, BAH
from holidays.countries.bangladesh import Bangladesh, BD, BGD
from holidays.countries.barbados import Barbados, BB, BRB
from holidays.countries.belarus import Belarus, BY, BLR
from holidays.countries.belgium import Belgium, BE, BEL
from holidays.countries.belize import Belize, BZ, BLZ
from holidays.countries.benin import Benin, BJ, BEN
from holidays.countries.bermuda import Bermuda, BM, BMU
from holidays.countries.bhutan import Bhutan, BT, BTN
from holidays.countries.bolivia import Bolivia, BO, BOL
from holidays.countries.bonaire_sint_eustatius_and_saba import BonaireSintEustatiusAndSaba, BQ, BES
from holidays.countries.bosnia_and_herzegovina import BosniaAndHerzegovina, BA, BIH
from holidays.countries.botswana import Botswana, BW, BWA
from holidays.countries.bouvet_island import BouvetIsland, BV, BVT
from holidays.countries.brazil import Brazil, BR, BRA
from holidays.countries.british_indian_ocean_territory import BritishIndianOceanTerritory, IO, IOT
from holidays.countries.british_virgin_islands import BritishVirginIslands, VG, VGB
from holidays.countries.brunei import Brunei, BN, BRN
from holidays.countries.bulgaria import Bulgaria, BG, BLG
from holidays.countries.burkina_faso import BurkinaFaso, BF, BFA
from holidays.countries.burundi import Burundi, BI, BDI
from holidays.countries.cabo_verde import CaboVerde, CV, CPV
from holidays.countries.cambodia import Cambodia, KH, KHM
from holidays.countries.cameroon import Cameroon, CM, CMR
from holidays.countries.canada import Canada, CA, CAN
from holidays.countries.cayman_islands import CaymanIslands, KY, CYM
from holidays.countries.central_african_republic import CentralAfricanRepublic, CF, CAF
from holidays.countries.chad import Chad, TD, TCD
from holidays.countries.chile import Chile, CL, CHL
from holidays.countries.china import China, CN, CHN
from holidays.countries.christmas_island import ChristmasIsland, CX, CXR
from holidays.countries.cocos_islands import CocosIslands, CC, CCK
from holidays.countries.colombia import Colombia, CO, COL
from holidays.countries.comoros import Comoros, KM, COM
from holidays.countries.congo import Congo, CG, COG
from holidays.countries.cook_islands import CookIslands, CK, COK
from holidays.countries.costa_rica import CostaRica, CR, CRI
from holidays.countries.croatia import Croatia, HR, HRV
from holidays.countries.cuba import Cuba, CU, CUB
from holidays.countries.curacao import Curacao, CW, CUW
from holidays.countries.cyprus import Cyprus, CY, CYP
from holidays.countries.czechia import Czechia, CZ, CZE
from holidays.countries.denmark import Denmark, DK, DNK
from holidays.countries.djibouti import Djibouti, DJ, DJI
from holidays.countries.dominica import Dominica, DM, DMA
from holidays.countries.dominican_republic import DominicanRepublic, DO, DOM
from holidays.countries.dr_congo import DRCongo, CD, COD
from holidays.countries.ecuador import Ecuador, EC, ECU
from holidays.countries.egypt import Egypt, EG, EGY
from holidays.countries.el_salvador import ElSalvador, SV, SLV
from holidays.countries.equatorial_guinea import EquatorialGuinea, GQ, GNQ
from holidays.countries.eritrea import Eritrea, ER, ERI
from holidays.countries.estonia import Estonia, EE, EST
from holidays.countries.eswatini import Eswatini, SZ, SZW, Swaziland
from holidays.countries.ethiopia import Ethiopia, ET, ETH
from holidays.countries.falkland_islands import FalklandIslands, FK, FLK
from holidays.countries.faroe_islands import FaroeIslands, FO, FRO
from holidays.countries.fiji import Fiji, FJ, FJI
from holidays.countries.finland import Finland, FI, FIN
from holidays.countries.france import France, FR, FRA
from holidays.countries.french_guiana import FrenchGuiana, GF, GUF, HolidaysGF
from holidays.countries.french_polynesia import FrenchPolynesia, PF, PYF, HolidaysPF
from holidays.countries.french_southern_territories import (
    FrenchSouthernTerritories,
    TF,
    ATF,
    HolidaysTF,
)
from holidays.countries.gabon import Gabon, GA, GAB
from holidays.countries.gambia import Gambia, GM, GMB
from holidays.countries.georgia import Georgia, GE, GEO
from holidays.countries.germany import Germany, DE, DEU
from holidays.countries.ghana import Ghana, GH, GHA
from holidays.countries.gibraltar import Gibraltar, GI, GIB
from holidays.countries.greece import Greece, GR, GRC
from holidays.countries.greenland import Greenland, GL, GRL
from holidays.countries.grenada import Grenada, GD, GRD
from holidays.countries.guadeloupe import Guadeloupe, GP, GLP, HolidaysGP
from holidays.countries.guam import Guam, GU, GUM, HolidaysGU
from holidays.countries.guatemala import Guatemala, GT, GUA
from holidays.countries.guernsey import Guernsey, GG, GGY
from holidays.countries.guinea import Guinea, GN, GIN
from holidays.countries.guinea_bissau import GuineaBissau, GW, GNB
from holidays.countries.guyana import Guyana, GY, GUY
from holidays.countries.haiti import Haiti, HT, HTI
from holidays.countries.heard_island_and_mcdonald_islands import (
    HeardIslandAndMcDonaldIslands,
    HM,
    HMD,
)
from holidays.countries.honduras import Honduras, HN, HND
from holidays.countries.hongkong import HongKong, HK, HKG
from holidays.countries.hungary import Hungary, HU, HUN
from holidays.countries.iceland import Iceland, IS, ISL
from holidays.countries.india import India, IN, IND
from holidays.countries.indonesia import Indonesia, ID, IDN
from holidays.countries.iran import Iran, IR, IRN
from holidays.countries.iraq import Iraq, IQ, IRQ
from holidays.countries.ireland import Ireland, IE, IRL
from holidays.countries.isle_of_man import IsleOfMan, IM, IMN
from holidays.countries.israel import Israel, IL, ISR
from holidays.countries.italy import Italy, IT, ITA
from holidays.countries.ivory_coast import IvoryCoast, CI, CIV
from holidays.countries.jamaica import Jamaica, JM, JAM
from holidays.countries.japan import Japan, JP, JPN
from holidays.countries.jersey import Jersey, JE, JEY
from holidays.countries.jordan import Jordan, JO, JOR
from holidays.countries.kazakhstan import Kazakhstan, KZ, KAZ
from holidays.countries.kenya import Kenya, KE, KEN
from holidays.countries.kiribati import Kiribati, KI, KIR
from holidays.countries.kosovo import Kosovo, XK, XKK
from holidays.countries.kuwait import Kuwait, KW, KWT
from holidays.countries.kyrgyzstan import Kyrgyzstan, KG, KGZ
from holidays.countries.laos import Laos, LA, LAO
from holidays.countries.latvia import Latvia, LV, LVA
from holidays.countries.lebanon import Lebanon, LB, LBN
from holidays.countries.lesotho import Lesotho, LS, LSO
from holidays.countries.liberia import Liberia, LR, LBR
from holidays.countries.libya import Libya, LY, LBY
from holidays.countries.liechtenstein import Liechtenstein, LI, LIE
from holidays.countries.lithuania import Lithuania, LT, LTU
from holidays.countries.luxembourg import Luxembourg, LU, LUX
from holidays.countries.macau import Macau, MO, MAC
from holidays.countries.madagascar import Madagascar, MG, MDG
from holidays.countries.malawi import Malawi, MW, MWI
from holidays.countries.malaysia import Malaysia, MY, MYS
from holidays.countries.maldives import Maldives, MV, MDV
from holidays.countries.mali import Mali, ML, MLI
from holidays.countries.malta import Malta, MT, MLT
from holidays.countries.marshall_islands import MarshallIslands, MH, MHL, HolidaysMH
from holidays.countries.martinique import Martinique, MQ, MTQ, HolidaysMQ
from holidays.countries.mauritania import Mauritania, MR, MRT
from holidays.countries.mauritius import Mauritius, MU, MUS
from holidays.countries.mayotte import Mayotte, YT, MYT, HolidaysYT
from holidays.countries.mexico import Mexico, MX, MEX
from holidays.countries.micronesia import Micronesia, FM, FSM
from holidays.countries.moldova import Moldova, MD, MDA
from holidays.countries.monaco import Monaco, MC, MCO
from holidays.countries.mongolia import Mongolia, MN, MNG
from holidays.countries.montenegro import Montenegro, ME, MNE
from holidays.countries.montserrat import Montserrat, MS, MSR
from holidays.countries.morocco import Morocco, MA, MOR
from holidays.countries.mozambique import Mozambique, MZ, MOZ
from holidays.countries.myanmar import Myanmar, MM, MMR
from holidays.countries.namibia import Namibia, NA, NAM
from holidays.countries.nauru import Nauru, NR, NRU
from holidays.countries.nepal import Nepal, NP, NPL
from holidays.countries.netherlands import Netherlands, NL, NLD
from holidays.countries.new_caledonia import NewCaledonia, NC, NCL, HolidaysNC
from holidays.countries.new_zealand import NewZealand, NZ, NZL
from holidays.countries.nicaragua import Nicaragua, NI, NIC
from holidays.countries.niger import Niger, NE, NER
from holidays.countries.nigeria import Nigeria, NG, NGA
from holidays.countries.niue import Niue, NU, NIU
from holidays.countries.norfolk_island import NorfolkIsland, NF, NFK
from holidays.countries.north_korea import NorthKorea, KP, PRK
from holidays.countries.north_macedonia import NorthMacedonia, MK, MKD
from holidays.countries.northern_mariana_islands import NorthernMarianaIslands, MP, MNP, HolidaysMP
from holidays.countries.norway import Norway, NO, NOR
from holidays.countries.oman import Oman, OM, OMN
from holidays.countries.pakistan import Pakistan, PK, PAK
from holidays.countries.palau import Palau, PW, PLW
from holidays.countries.palestine import Palestine, PS, PSE
from holidays.countries.panama import Panama, PA, PAN
from holidays.countries.papua_new_guinea import PapuaNewGuinea, PG, PNG
from holidays.countries.paraguay import Paraguay, PY, PRY
from holidays.countries.peru import Peru, PE, PER
from holidays.countries.philippines import Philippines, PH, PHL
from holidays.countries.pitcairn_islands import PitcairnIslands, PN, PCN
from holidays.countries.poland import Poland, PL, POL
from holidays.countries.portugal import Portugal, PT, PRT
from holidays.countries.puerto_rico import PuertoRico, PR, PRI, HolidaysPR
from holidays.countries.qatar import Qatar, QA, QAT
from holidays.countries.reunion import Reunion, RE, REU, HolidaysRE
from holidays.countries.romania import Romania, RO, ROU
from holidays.countries.russia import Russia, RU, RUS
from holidays.countries.rwanda import Rwanda, RW, RWA
from holidays.countries.saint_barthelemy import SaintBarthelemy, BL, BLM, HolidaysBL
from holidays.countries.saint_helena_ascension_and_tristan_da_cunha import (
    SaintHelenaAscensionAndTristanDaCunha,
    SH,
    SHN,
)
from holidays.countries.saint_kitts_and_nevis import SaintKittsAndNevis, KN, KNA
from holidays.countries.saint_lucia import SaintLucia, LC, LCA
from holidays.countries.saint_martin import SaintMartin, MF, MAF, HolidaysMF
from holidays.countries.saint_pierre_and_miquelon import (
    SaintPierreAndMiquelon,
    PM,
    SPM,
    HolidaysPM,
)
from holidays.countries.saint_vincent_and_the_grenadines import (
    SaintVincentAndTheGrenadines,
    VC,
    VCT,
)
from holidays.countries.samoa import Samoa, WS, WSM
from holidays.countries.san_marino import SanMarino, SM, SMR
from holidays.countries.sao_tome_and_principe import SaoTomeAndPrincipe, ST, STP
from holidays.countries.saudi_arabia import SaudiArabia, SA, SAU
from holidays.countries.senegal import Senegal, SN, SEN
from holidays.countries.serbia import Serbia, RS, SRB
from holidays.countries.seychelles import Seychelles, SC, SYC
from holidays.countries.sierra_leone import SierraLeone, SL, SLE
from holidays.countries.singapore import Singapore, SG, SGP
from holidays.countries.sint_maarten import SintMaarten, SX, SXM
from holidays.countries.slovakia import Slovakia, SK, SVK
from holidays.countries.slovenia import Slovenia, SI, SVN
from holidays.countries.solomon_islands import SolomonIslands, SB, SLB
from holidays.countries.somalia import Somalia, SO, SOM
from holidays.countries.south_africa import SouthAfrica, ZA, ZAF
from holidays.countries.south_georgia_and_the_south_sandwich_islands import (
    SouthGeorgiaAndTheSouthSandwichIslands,
    GS,
    SGS,
)
from holidays.countries.south_korea import SouthKorea, KR, KOR, Korea
from holidays.countries.south_sudan import SouthSudan, SS, SSD
from holidays.countries.spain import Spain, ES, ESP
from holidays.countries.sri_lanka import SriLanka, LK, LKA
from holidays.countries.sudan import Sudan, SD, SDN
from holidays.countries.suriname import Suriname, SR, SUR
from holidays.countries.svalbard_and_jan_mayen import SvalbardAndJanMayen, SJ, SJM, HolidaysSJ
from holidays.countries.sweden import Sweden, SE, SWE
from holidays.countries.switzerland import Switzerland, CH, CHE
from holidays.countries.syrian_arab_republic import SyrianArabRepublic, SY, SYR
from holidays.countries.taiwan import Taiwan, TW, TWN
from holidays.countries.tajikistan import Tajikistan, TJ, TJK
from holidays.countries.tanzania import Tanzania, TZ, TZA
from holidays.countries.thailand import Thailand, TH, THA
from holidays.countries.timor_leste import TimorLeste, TL, TLS
from holidays.countries.togo import Togo, TG, TGO
from holidays.countries.tokelau import Tokelau, TK, TKL
from holidays.countries.tonga import Tonga, TO, TON
from holidays.countries.trinidad_and_tobago import TrinidadAndTobago, TT, TTO
from holidays.countries.tunisia import Tunisia, TN, TUN
from holidays.countries.turkey import Turkey, TR, TUR
from holidays.countries.turkmenistan import Turkmenistan, TM, TKM
from holidays.countries.turks_and_caicos_islands import TurksAndCaicosIslands, TC, TCA
from holidays.countries.tuvalu import Tuvalu, TV, TUV
from holidays.countries.uganda import Uganda, UG, UGA
from holidays.countries.ukraine import Ukraine, UA, UKR
from holidays.countries.united_arab_emirates import UnitedArabEmirates, AE, ARE
from holidays.countries.united_kingdom import UnitedKingdom, GB, GBR, UK
from holidays.countries.united_states import UnitedStates, US, USA
from holidays.countries.united_states_minor_outlying_islands import (
    UnitedStatesMinorOutlyingIslands,
    UM,
    UMI,
    HolidaysUM,
)
from holidays.countries.united_states_virgin_islands import (
    UnitedStatesVirginIslands,
    VI,
    VIR,
    HolidaysVI,
)
from holidays.countries.uruguay import Uruguay, UY, URY
from holidays.countries.uzbekistan import Uzbekistan, UZ, UZB
from holidays.countries.vanuatu import Vanuatu, VU, VTU
from holidays.countries.vatican_city import VaticanCity, VA, VAT
from holidays.countries.venezuela import Venezuela, VE, VEN
from holidays.countries.vietnam import Vietnam, VN, VNM
from holidays.countries.wallis_and_futuna import WallisAndFutuna, WF, WLF, HolidaysWF
from holidays.countries.western_sahara import WesternSahara, EH, ESH
from holidays.countries.yemen import Yemen, YE, YEM
from holidays.countries.zambia import Zambia, ZM, ZMB
from holidays.countries.zimbabwe import Zimbabwe, ZW, ZWE



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars import _CustomIslamicHolidays
from holidays.calendars.gregorian import JAN, MAR, NOV, DEC
from holidays.groups import (
    BurmeseCalendarHolidays,
    ChineseCalendarHolidays,
    ChristianHolidays,
    InternationalHolidays,
    IslamicHolidays,
    StaticHolidays,
)
from holidays.holiday_base import HolidayBase


class Myanmar(
    HolidayBase,
    BurmeseCalendarHolidays,
    ChineseCalendarHolidays,
    ChristianHolidays,
    InternationalHolidays,
    IslamicHolidays,
    StaticHolidays,
):
    """Myanmar holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Myanmar>
        * <https://my.wikipedia.org/wiki/မြန်မာနိုင်ငံရှိ_အားလပ်ရက်များ>
        * [Algorithm, Program and Calculation of Myanmar Calendar](https://web.archive.org/web/20250510011425/http://cool-emerald.blogspot.com/2013/06/algorithm-program-and-calculation-of.html)
        * [2025 International and Chinese New Year](https://web.archive.org/web/20240709104600/https://www.moi.gov.mm/news/58594)
        * [Armed Forces Day](https://web.archive.org/web/20250522010622/https://burma.irrawaddy.com/on-this-day/2019/03/27/187691.html)
        * [2021-2024](https://web.archive.org/web/20241112125406/https://evisa.moip.gov.mm/home/publicholiday)
        * [2013-2025 (Webarchive)](https://web.archive.org/web/20250711015428/http://www.myanmarembassy.sg/contact-us/holidays/)
    """

    country = "MM"
    default_language = "my"
    # %s (estimated).
    estimated_label = tr("%s (ခန့်မှန်း)")
    supported_languages = ("en_US", "my", "th")
    # Myanmar gained its independence from British rule on January 4, 1948.
    start_year = 1948

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.

        In Myanmar, the dates of the Islamic calendar usually fall a day later than
        the corresponding dates in the Umm al-Qura calendar.
        """
        BurmeseCalendarHolidays.__init__(self)
        ChineseCalendarHolidays.__init__(self)
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self,
            cls=MyanmarIslamicHolidays,
            show_estimated=islamic_show_estimated,
            calendar_delta_days=+1,
        )
        StaticHolidays.__init__(self, cls=MyanmarStaticHolidays)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        if self._year >= 2025:
            # New Year's Day.
            self._add_new_years_day(tr("နိုင်ငံတကာနှစ်သစ်ကူးနေ့"))

        # Independence Day.
        self._add_holiday_jan_4(tr("လွတ်လပ်ရေးနေ့"))

        # Union Day.
        self._add_holiday_feb_12(tr("ပြည်ထောင်စုနေ့"))

        # Peasants' Day.
        name = tr("တောင်သူလယ်သမားနေ့")
        if self._year >= 1965:
            self._add_holiday_mar_2(name)
        elif self._year >= 1963:
            self._add_holiday_jan_1(name)

        self._add_holiday_mar_27(
            # Armed Forces Day.
            tr("တပ်မတော်နေ့")
            if self._year >= 1955
            # Revolution Day.
            else tr("တော်လှန်ရေးနေ့")
        )

        # May Day.
        self._add_labor_day(tr("မေဒေးနေ့"))

        # Martyrs' Day.
        self._add_holiday_jul_19(tr("အာဇာနည်နေ့"))

        # Christmas Day.
        self._add_christmas_day(tr("ခရစ္စမတ်နေ့"))

        # Eid al-Adha.
        self._add_eid_al_adha_day(tr("အီဒုလ်အဿွဟာနေ့"))

        if self._year >= 2025:
            # Chinese New Year.
            self._add_chinese_new_years_day(tr("တရုတ်နှစ်သစ်ကူးနေ့"))

        # Myanmar New Year.
        name = tr("မြန်မာနှစ်သစ်ကူး ရုံးပိတ်ရက်များ")
        if self._year >= 2024:
            self._add_myanmar_new_year(name, extra_days_after=8)
        elif self._year >= 2022:
            self._add_myanmar_new_year(name, extra_days_before=4)
        elif 2007 <= self._year <= 2016:
            self._add_myanmar_new_year(name, extra_days_before=1, extra_days_after=8)
        else:
            self._add_myanmar_new_year(name)

        # Full Moon Day of Tabaung.
        self._add_tabaung_full_moon_day(tr("တပေါင်းလပြည့်နေ့"))

        # Full Moon Day of Kason.
        self._add_kason_full_moon_day(tr("ကဆုန်လပြည့်နေ့"))

        # Full Moon Day of Waso.
        self._add_waso_full_moon_day(tr("ဝါဆိုလပြည့်နေ့"))

        # Thadingyut Holidays.
        name = tr("သီတင်းကျွတ်ပိတ်ရက်များ")
        self._add_thadingyut_full_moon_eve(name)
        self._add_thadingyut_full_moon_day(name)
        self._add_thadingyut_full_moon_day_two(name)

        # Diwali.
        self._add_myanmar_diwali(tr("ဒီပါဝလီနေ့"))

        # Full Moon Day of Tazaungmon.
        self._add_tazaungmon_full_moon_day(tr("တန်ဆောင်တိုင်လပြည့်နေ့"))

        # National Day.
        self._add_myanmar_national_day(tr("အမျိုးသားနေ့"))

        # Karen New Year.
        self._add_karen_new_year(tr("ကရင်နှစ်သစ်ကူးနေ့"))


class MM(Myanmar):
    pass


class MMR(Myanmar):
    pass


class MyanmarIslamicHolidays(_CustomIslamicHolidays):
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2020, 2025)


class MyanmarStaticHolidays:
    """Myanmar special holidays.

    References:
        * [2025](https://web.archive.org/web/20241221214023/https://www.gnlm.com.mm/govt-designates-continuous-public-days-for-2024-2025-to-ease-travel-and-extend-leisure/)
    """

    # Substituted date format.
    substituted_date_format = tr("%d-%m-%Y")

    # Day off (substituted from %s).
    substituted_label = tr("အလုပ်ပိတ်ရက် (%s မှ ပြန်လဲထားသည်)")

    special_public_holidays = {
        2024: (DEC, 31, JAN, 11, 2025),
        2025: (
            (MAR, 12, MAR, 22),
            (MAR, 14, MAR, 29),
            (NOV, 3, NOV, 8),
            (DEC, 26, JAN, 3, 2026),
        ),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from datetime import date

from holidays.calendars.buddhist import _BuddhistLunisolar
from holidays.groups.eastern import EasternCalendarHolidays


class BuddhistCalendarHolidays(EasternCalendarHolidays):
    """
    Buddhist lunisolar calendar holidays.
    """

    def __init__(self, cls=None, *, show_estimated=False) -> None:
        self._buddhist_calendar = cls() if cls else _BuddhistLunisolar()
        self._buddhist_calendar_show_estimated = show_estimated

    def _add_buddhist_calendar_holiday(
        self, name: str, dt_estimated: tuple[date | None, bool]
    ) -> date | None:
        """
        Add Buddhist calendar holiday.

        Adds customizable estimation label to holiday name if holiday date
        is an estimation.
        """
        return self._add_eastern_calendar_holiday(
            name, dt_estimated, show_estimated=self._buddhist_calendar_show_estimated
        )

    def _add_vesak(self, name) -> date | None:
        """
        Add Vesak (15th day of the 4th lunar month).

        Vesak for Thailand, Laos, Singapore and Indonesia.
        https://en.wikipedia.org/wiki/Vesak
        """
        return self._add_buddhist_calendar_holiday(
            name, self._buddhist_calendar.vesak_date(self._year)
        )

    def _add_vesak_may(self, name) -> date | None:
        """
        Add Vesak (on the day of the first full moon in May
        in the Gregorian calendar).

        Vesak for Sri Lanka, Nepal, India, Bangladesh and Malaysia.
        https://en.wikipedia.org/wiki/Vesak
        """
        return self._add_buddhist_calendar_holiday(
            name, self._buddhist_calendar.vesak_may_date(self._year)
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    MON_TO_NEXT_TUE,
    SUN_TO_PREV_SAT,
    SUN_TO_NEXT_MON,
)


class Aruba(ObservedHolidayBase, ChristianHolidays, InternationalHolidays):
    """Aruba holidays.

    References:
        * [AB 2013 no. 14](https://web.archive.org/web/20250212184919/https://cuatro.sim-cdn.nl/arubaoverheid2858bd/uploads/ab2013no.14_0.pdf?cb=WDbZKYCl)
        * [Holidays List (English)]( https://web.archive.org/web/20230808172049/https://www.government.aw/information-public-services/hiring-people_47940/item/holidays_43823.html)
        * [Holidays List (Dutch)](https://web.archive.org/web/20231208145916/https://www.overheid.aw/informatie-dienstverlening/ondernemen-en-werken-subthemas_46970/item/feestdagen_37375.html)
        * [Holidays List (Papiamento)](https://web.archive.org/web/20231202011228/https://www.gobierno.aw/informacion-tocante-servicio/haci-negoshi-y-traha-sub-topics_47789/item/dia-di-fiesta_41242.html)
        * [National Holidays & Celebrations](https://web.archive.org/web/20250210000132/https://www.visitaruba.com/about-aruba/national-holidays-and-celebrations/)
        * <https://web.archive.org/web/20240619235841/https://www.arubatoday.com/we-celebrate-our-national-hero-betico-croes/>
        * <https://web.archive.org/web/20240721173750/https://caribbeannewsglobal.com/carnival-monday-remains-a-festive-day-in-aruba/>
        * <https://web.archive.org/web/20250426210119/https://www.aruba.com/us/calendar/national-anthem-and-flag-day>
    """

    country = "AW"
    default_language = "pap_AW"
    supported_languages = ("en_US", "nl", "pap_AW", "uk")
    # The Netherlands Antilles was established on December 15th, 1954.
    start_year = 1955

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        kwargs.setdefault("observed_rule", SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # Aña Nobo.
        # Status: In-Use.

        # New Year's Day.
        self._add_new_years_day(tr("Aña Nobo"))

        # Dia Di Betico.
        # Status: In-Use.
        # Started in 1989.

        if self._year >= 1989:
            # Betico Day.
            self._add_holiday_jan_25(tr("Dia di Betico"))

        # Dialuna prome cu diaranson di shinish.
        # Status: In-Use.
        # Starts as a public holiday from 1956 onwards.
        # Event cancelled but remain a holiday in 2021.
        # Have its name changed from 2023 onwards.

        if self._year >= 1956:
            self._add_ash_monday(
                # Carnival Monday.
                tr("Dialuna despues di Carnaval Grandi")
                if self._year <= 2022
                # Monday before Ash Wednesday.
                else tr("Dialuna prome cu diaranson di shinish")
            )

        # Dia di Himno y Bandera.
        # Status: In-Use.
        # Started in 1976.

        if self._year >= 1976:
            # National Anthem and Flag Day.
            self._add_holiday_mar_18(tr("Dia di Himno y Bandera"))

        # Bierna Santo.
        # Status: In-Use.

        # Good Friday.
        self._add_good_friday(tr("Bierna Santo"))

        # Di dos dia di Pasco di Resureccion.
        # Status: In-Use.

        # Easter Monday.
        self._add_easter_monday(tr("Di dos dia di Pasco di Resureccion"))

        # Dia di Labor/Dia di Obrero.
        # Status: In-Use.
        # If fall on Sunday, then this will be move to next working day.
        # This is placed here before King's/Queen's Day for _move_holiday logic.

        self._move_holiday(
            # Labor Day.
            self._add_labor_day(tr("Dia di Obrero")),
            rule=SUN_TO_NEXT_MON if self._year >= 1980 else MON_TO_NEXT_TUE + SUN_TO_NEXT_MON,
        )

        # Aña di La Reina/Aña di Rey/Dia di Rey.
        # Status: In-Use.
        # Started under Queen Wilhelmina in 1891.
        # Queen Beatrix kept Queen Juliana's Birthday after her coronation.
        # Switched to Aña di Rey in 2014 for King Willem-Alexander.
        # Have its name changed again to Dia di Rey from 2021 onwards.

        name = (
            # King's Day.
            tr("Dia di Rey")
            if self._year >= 2021
            else (
                tr("Aña di Rey")  # King's Day.
                if self._year >= 2014
                else tr("Aña di La Reina")  # Queen's Day.
            )
        )
        self._move_holiday(
            self._add_holiday_apr_27(name)
            if self._year >= 2014
            else self._add_holiday_apr_30(name),
            rule=SUN_TO_PREV_SAT if self._year >= 1980 else SUN_TO_NEXT_MON,
        )

        # Dia di Asuncion.
        # Status: In-Use.

        # Ascension Day.
        self._add_ascension_thursday(tr("Dia di Asuncion"))

        # Pasco di Nacemento.
        # Status: In-Use.

        # Christmas Day.
        self._add_christmas_day(tr("Pasco di Nacemento"))

        # Di dos dia di Pasco di Nacemento.
        # Status: In-Use.

        # Second Day of Christmas.
        self._add_christmas_day_two(tr("Di dos dia di Pasco di Nacemento"))


class AW(Aruba):
    pass


class ABW(Aruba):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars import _CustomIslamicHolidays
from holidays.calendars.gregorian import APR, MAY, JUN, JUL, AUG, SEP, OCT, NOV, DEC
from holidays.groups import ChristianHolidays, InternationalHolidays, IslamicHolidays
from holidays.observed_holiday_base import ObservedHolidayBase, SAT_SUN_TO_NEXT_MON


class Gambia(ObservedHolidayBase, ChristianHolidays, InternationalHolidays, IslamicHolidays):
    """Gambia holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_the_Gambia>
        * <https://web.archive.org/web/20230610113952/https://www.visitthegambia.gm/public-holidays/>
        * <https://web.archive.org/web/20250803084358/https://www.op.gov.gm/media-advisory-public-holidays>
    """

    country = "GM"
    default_language = "en_GM"
    # %s (estimated).
    estimated_label = tr("%s (estimated)")
    # %s (observed).
    observed_label = tr("%s (observed)")
    # %s (observed, estimated).
    observed_estimated_label = tr("%s (observed, estimated)")
    start_year = 1966
    supported_languages = ("en_GM", "en_US")

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self, cls=GambiaIslamicHolidays, show_estimated=islamic_show_estimated
        )
        kwargs.setdefault("observed_rule", SAT_SUN_TO_NEXT_MON)
        kwargs.setdefault("observed_since", 2021)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_observed(self._add_new_years_day(tr("New Year's Day")))

        # Independence Day.
        self._add_observed(self._add_holiday_feb_18(tr("Independence Day")))

        # Good Friday.
        self._add_good_friday(tr("Good Friday"))

        # Easter Monday.
        self._add_easter_monday(tr("Easter Monday"))

        # Labor Day.
        self._add_observed(self._add_labor_day(tr("Labour Day")))

        # Africa Liberation Day.
        self._add_observed(self._add_africa_day(tr("Africa Liberation Day")))

        # July 22 Revolution Day.
        self._add_observed(self._add_holiday_jul_22(tr("July 22 Revolution Day")))

        # Assumption Day.
        self._add_observed(self._add_assumption_of_mary_day(tr("Feast of the Assumption")))

        # Christmas Day.
        self._add_christmas_day(tr("Christmas Day"))

        # Boxing Day.
        self._add_observed(self._add_christmas_day_two(tr("Boxing Day")))

        # Ashura.
        for dt in self._add_ashura_day(tr("Yawmul Ashura")):
            self._add_observed(dt)

        # Prophet's Birthday.
        for dt in self._add_mawlid_day(tr("Mawlid Nabi")):
            self._add_observed(dt)

        # Laylat al-Qadr.
        for dt in self._add_laylat_al_qadr_day(tr("Lialat-Ul-Qadr")):
            self._add_observed(dt)

        # Eid al-Fitr.
        name = tr("Koriteh")
        self._add_eid_al_fitr_day(name)
        if self._year >= 2021:
            for dt in self._add_eid_al_fitr_day_two(name):
                self._add_observed(dt)

        # Eid al-Adha.
        name = tr("Tobaski")
        self._add_eid_al_adha_day(name)
        if self._year >= 2021:
            for dt in self._add_eid_al_adha_day_two(name):
                self._add_observed(dt)


class GM(Gambia):
    pass


class GMB(Gambia):
    pass


class GambiaIslamicHolidays(_CustomIslamicHolidays):
    # https://web.archive.org/web/20240716225449/https://www.timeanddate.com/holidays/gambia/ashura
    ASHURA_DATES_CONFIRMED_YEARS = (2015, 2025)
    ASHURA_DATES = {
        2015: (OCT, 24),
        2016: (OCT, 12),
        2017: (OCT, 1),
        2019: (SEP, 10),
        2021: (AUG, 19),
    }

    # https://web.archive.org/web/20241209170416/https://www.timeanddate.com/holidays/gambia/eid-al-adha
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2015, 2025)
    EID_AL_ADHA_DATES = {
        2015: (SEP, 24),
        2016: (SEP, 13),
        2017: (SEP, 2),
        2018: (AUG, 22),
    }

    # https://web.archive.org/web/20241204114244/https://www.timeanddate.com/holidays/gambia/eid-al-fitr
    EID_AL_FITR_DATES_CONFIRMED_YEARS = (2015, 2025)
    EID_AL_FITR_DATES = {
        2015: (JUL, 18),
        2016: (JUL, 7),
        2017: (JUN, 26),
        2020: (MAY, 23),
    }

    # https://web.archive.org/web/20241209070551/https://www.timeanddate.com/holidays/gambia/laylat-al-qadr
    LAYLAT_AL_QADR_DATES_CONFIRMED_YEARS = (2015, 2025)
    LAYLAT_AL_QADR_DATES = {
        2016: (JUL, 3),
        2022: (APR, 29),
    }

    # https://web.archive.org/web/20240909111449/https://www.timeanddate.com/holidays/gambia/prophet-birthday
    MAWLID_DATES_CONFIRMED_YEARS = (2016, 2024)
    MAWLID_DATES = {
        2016: (DEC, 12),
        2017: (DEC, 1),
        2018: (NOV, 21),
        2021: (OCT, 19),
        2023: (SEP, 28),
        2024: (SEP, 16),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import MAY
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.observed_holiday_base import ObservedHolidayBase, SUN_TO_PREV_SAT


class BonaireSintEustatiusAndSaba(
    ObservedHolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays
):
    """Bonaire, Sint Eustatius and Saba holidays.

    References:
        * [Public Holidays 2025 (English)](https://web.archive.org/web/20250620050728/https://english.rijksdienstcn.com/social-affairs-work/employment/official-holidays)
        * [Public Holidays 2025 (Dutch)](https://web.archive.org/web/20250323122427/https://www.rijksdienstcn.com/sociale-zaken-werk/arbeid/officiele-feestdagen)
        * [Public Holidays 2025 (Papiamento)](https://web.archive.org/web/20250323125431/https://papiamentu.rijksdienstcn.com/asuntunan-sosial-i-labor/labor/dia-di-fiesta-ofisial)
        * [Arbeidsregeling 2000](https://web.archive.org/web/20250625203706/https://lokaleregelgeving.overheid.nl/CVDR10375/1)
        * [Dia di Rincon 2025](https://web.archive.org/web/20250515183744/https://sunwisebonaire.com/blog/dia-di-rincon-2025-on-bonaire/)
    """

    country = "BQ"
    default_language = "nl"
    # Became special municipalities of the Netherlands on October 10th, 2010.
    start_year = 2011
    subdivisions = (
        "BON",  # Bonaire.
        "SAB",  # Saba.
        "STA",  # Sint Eustatius.
    )
    subdivisions_aliases = {
        "Bonaire": "BON",
        "Saba": "SAB",
        "Sint Eustatius": "STA",
    }
    supported_languages = ("en_BQ", "en_US", "nl", "pap_BQ")

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, cls=BonaireSintEustatiusAndSabaStaticHolidays)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("Nieuwjaarsdag"))

        # Good Friday.
        self._add_good_friday(tr("Goede Vrijdag"))

        # Easter Sunday.
        self._add_easter_sunday(tr("Eerste Paasdag"))

        # Easter Monday.
        self._add_easter_monday(tr("Tweede Paasdag"))

        self._move_holiday(
            # King's Day.
            self._add_holiday_apr_27(tr("Koningsdag"))
            if self._year >= 2014
            # Queen's Day.
            else self._add_holiday_apr_30(tr("Koninginnedag")),
            rule=SUN_TO_PREV_SAT,
        )

        # Labor Day.
        self._add_labor_day(tr("Dag van de Arbeid"))

        # Ascension Day.
        self._add_ascension_thursday(tr("Hemelvaartsdag"))

        # Whit Sunday.
        self._add_whit_sunday(tr("Eerste Pinksterdag"))

        # Christmas Day.
        self._add_christmas_day(tr("Eerste Kerstdag"))

        # Second Day of Christmas.
        self._add_christmas_day_two(tr("Tweede Kerstdag"))

    def _populate_subdiv_bon_public_holidays(self):
        if self._year >= 2020:
            # Rincon Day.
            self._add_holiday_apr_30(tr("Rincondag"))

        # Bonaire Day.
        self._add_holiday_sep_6(tr("Bonairedag"))

    def _populate_subdiv_sab_public_holidays(self):
        # Carnival Monday.
        self._add_holiday_last_mon_of_jul(tr("Dag na de carnavalsoptocht"))

        # Saba Day.
        self._add_holiday_1st_fri_of_dec(tr("Sabadag"))

    def _populate_subdiv_sta_public_holidays(self):
        if self._year >= 2022:
            # Emancipation Day.
            self._add_holiday_jul_1(tr("Emancipatiedag"))

        # Statia Day.
        self._add_holiday_nov_16(tr("Statiadag"))


class BQ(BonaireSintEustatiusAndSaba):
    pass


class BES(BonaireSintEustatiusAndSaba):
    pass


class BonaireSintEustatiusAndSabaStaticHolidays(StaticHolidays):
    """Bonaire, Sint Eustatius and Saba special holidays.

    References:
        * [Bonaire May 2nd, 2025](https://web.archive.org/web/20250620052258/https://bonairegov.com/nieuwsoverzicht/artikel/vrijdag-2-mei-2025-aangewezen-als-vrije-dag-voor-ambtenaren)
    """

    special_bon_public_holidays = {
        # Bridge Holiday.
        2025: (MAY, 2, tr("Brugdag")),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.holiday_base import HolidayBase


class Samoa(HolidayBase, ChristianHolidays, InternationalHolidays):
    """Samoa holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Samoa>
        * <https://web.archive.org/web/20250407135339/https://www.timeanddate.com/holidays/samoa/>
        * <https://web.archive.org/web/20240914112255/https://www.mcil.gov.ws/?attachment_id=6336>
        * <https://web.archive.org/web/20250413020419/http://www.paclii.org/ws/legis/consol_act_2020/pha2008163/>
    """

    country = "WS"

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)

        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day("New Year's Day")
        self._add_new_years_day_two("The Day After New Year's Day")

        # Good Friday.
        self._add_good_friday("Good Friday")
        self._add_holy_saturday("Day After Good Friday")

        # Easter Monday.
        self._add_easter_monday("Easter Monday")

        # Mother's Day.
        self._add_holiday_1_day_past_2nd_sun_of_may("Mother's Day")

        # Independence Day.
        self._add_holiday_jun_1("Independence Day")

        # Father's Day.
        self._add_holiday_1_day_past_2nd_sun_of_aug("Father's Day")

        # White Monday (Lotu a Tamaiti).
        self._add_holiday_1_day_past_2nd_sun_of_oct("White Sunday (Lotu a Tamaiti)")

        # Christmas Day.
        self._add_christmas_day("Christmas Day")

        # Boxing Day.
        self._add_christmas_day_two("Boxing Day")


class WS(Samoa):
    pass


class WSM(Samoa):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from holidays.financial.shanghai_stock_exchange import ShanghaiStockExchange


class ShenzhenStockExchange(ShanghaiStockExchange):
    """Shenzhen Stock Exchange (SZSE) holidays.

    Shenzhen Stock Exchange publishes the same annual holiday schedule as
    Shanghai Stock Exchange, so this class reuses the Shanghai calendar
    implementation and provides SZSE-specific market codes and translations.

    References:
        * [2024](http://archive.today/2026.03.30-044431/https://www.szse.cn/disclosure/notice/t20231226_605108.html)
        * [2025](http://archive.today/2026.03.30-044217/https://www.szse.cn/www/disclosure/notice/general/t20241223_611283.html)
        * [2026](http://archive.today/2026.03.30-043931/https://www.szse.cn/disclosure/notice/t20251222_618087.html)
    """

    market = "XSHE"


class XSHE(ShenzhenStockExchange):
    pass


class SZSE(ShenzhenStockExchange):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars import _CustomIslamicHolidays, _CustomHinduHolidays
from holidays.calendars.gregorian import JAN, FEB, MAR, APR, MAY, AUG, SEP, OCT, NOV
from holidays.constants import PUBLIC, WORKDAY
from holidays.groups import (
    ChristianHolidays,
    HinduCalendarHolidays,
    InternationalHolidays,
    IslamicHolidays,
    StaticHolidays,
)
from holidays.holiday_base import HolidayBase


class Nepal(
    HolidayBase,
    ChristianHolidays,
    HinduCalendarHolidays,
    InternationalHolidays,
    IslamicHolidays,
    StaticHolidays,
):
    """Nepal holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Nepal>
        * <https://web.archive.org/web/20250422191700/https://narayanilawfirm.org.np/list-of-public-holidays-in-nepal-2079/>
        * <https://web.archive.org/web/20250131082346/https://www.timeanddate.com/holidays/nepal/>
        * <https://web.archive.org/web/20240911172745/https://kathmandupost.com/national/2018/03/29/government-trims-public-holidays-by-22-days>
        * <https://web.archive.org/web/20250419211438/https://kathmandupost.com/national/2021/02/13/two-years-after-shortening-public-holiday-list-government-starts-adding-to-it-again>
        * <https://web.archive.org/web/20250316094224/https://english.hamropatro.com/nepali-public-holidays>
        * <https://web.archive.org/web/20250527164012/https://english.hamropatro.com/calendar>
        * <https://web.archive.org/web/20250322000610/https://www.ashesh.com.np/nepali-calendar/>
        * <https://web.archive.org/web/20250117021312/https://www.bolpatra.gov.np/egp/openDateConverter>
        * <https://web.archive.org/web/20241013164133/https://www.moha.gov.np/en/page/holidays>
    """

    country = "NP"
    default_language = "ne"
    # %s (estimated).
    estimated_label = tr("%s (अनुमानित)")
    start_year = 2010
    supported_categories = (PUBLIC, WORKDAY)
    supported_languages = ("en_US", "ne")

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.

        In Nepal, the dates of the Islamic calendar usually fall a day later than
        the corresponding dates in the Umm al-Qura calendar.
        """
        ChristianHolidays.__init__(self)
        HinduCalendarHolidays.__init__(self, cls=NepalHinduHolidays)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self,
            cls=NepalIslamicHolidays,
            show_estimated=islamic_show_estimated,
            calendar_delta_days=+1,
        )
        StaticHolidays.__init__(self, cls=NepalStaticHolidays)
        super().__init__(*args, **kwargs)

    def _add_non_continuous_holidays(self, *, is_workday: bool = False):
        """Holidays removed by MoHA between 2019-2020."""
        if (2019 <= self._year <= 2020) == is_workday:
            martyrs_day_dates = {
                2010: (JAN, 30),
                2011: (JAN, 30),
                2012: (JAN, 30),
                2013: (JAN, 29),
                2014: (JAN, 30),
                2015: (JAN, 30),
                2016: (JAN, 30),
                2017: (JAN, 29),
                2018: (JAN, 30),
                2019: (JAN, 30),
                2020: (JAN, 30),
                2021: (JAN, 29),
                2022: (JAN, 30),
                2023: (JAN, 30),
                2024: (JAN, 30),
                2025: (JAN, 29),
                2026: (JAN, 29),
                2027: (JAN, 29),
                2028: (JAN, 29),
                2029: (JAN, 29),
                2030: (JAN, 29),
                2031: (JAN, 30),
                2032: (JAN, 30),
                2033: (JAN, 29),
            }
            if dt := martyrs_day_dates.get(self._year):
                # Martyr's Day.
                self._add_holiday(tr("शहीद दिवस"), dt)

            democracy_day_dates = {
                2010: (FEB, 19),
                2011: (FEB, 19),
                2012: (FEB, 19),
                2013: (FEB, 18),
                2014: (FEB, 19),
                2015: (FEB, 19),
                2016: (FEB, 19),
                2017: (FEB, 18),
                2018: (FEB, 19),
                2019: (FEB, 19),
                2020: (FEB, 19),
                2021: (FEB, 19),
                2022: (FEB, 19),
                2023: (FEB, 19),
                2024: (FEB, 19),
                2025: (FEB, 19),
                2026: (FEB, 19),
                2027: (FEB, 19),
                2028: (FEB, 19),
                2029: (FEB, 19),
                2030: (FEB, 19),
                2031: (FEB, 20),
                2032: (FEB, 20),
                2033: (FEB, 19),
            }
            if dt := democracy_day_dates.get(self._year):
                # National Democracy Day.
                self._add_holiday(tr("राष्ट्रिय प्रजातन्त्र दिवस"), dt)

            republic_day_dates = {
                2010: (MAY, 29),
                2011: (MAY, 29),
                2012: (MAY, 28),
                2013: (MAY, 29),
                2014: (MAY, 29),
                2015: (MAY, 29),
                2016: (MAY, 28),
                2017: (MAY, 29),
                2018: (MAY, 29),
                2019: (MAY, 29),
                2020: (MAY, 28),
                2021: (MAY, 29),
                2022: (MAY, 29),
                2023: (MAY, 29),
                2024: (MAY, 28),
                2025: (MAY, 28),
                2026: (MAY, 29),
                2027: (MAY, 29),
                2028: (MAY, 28),
                2029: (MAY, 28),
                2030: (MAY, 29),
                2031: (MAY, 29),
                2032: (MAY, 28),
            }
            if dt := republic_day_dates.get(self._year):
                # Established in 2009.

                # Republic Day.
                self._add_holiday(tr("गणतन्त्र दिवस"), dt)

            # Hindu Holidays.

            # Ram Navami.
            self._add_ram_navami(tr("राम नवमी"))

            # Janai Poornima.
            self._add_raksha_bandhan(tr("जनै पूर्णिमा"))

            # Shree Krishna Janmashtami.
            self._add_janmashtami(tr("श्रीकृष्ण जन्माष्टमी"))

            # Ghatasthapana.
            self._add_sharad_navratri(tr("घटस्थापना"))

            # Duwadashi (Dashain).
            self._add_papankusha_duwadashi(tr("दुवादशी (दशैं)"))

    def _populate_public_holidays(self):
        if self._year >= 2023:
            # Prithvi Jayanti.
            self._add_holiday_jan_11(tr("पृथ्वी जयन्ती"))

        # International Women's Day.
        self._add_womens_day(tr("अन्तर्राष्ट्रिय महिला दिवस"))

        # Nepali New Year.
        self._add_vaisakhi(tr("नेपाली नयाँ वर्ष"))

        # International Labor Day.
        self._add_labor_day(tr("अन्तर्राष्ट्रिय श्रम दिवस"))

        constitution_day_dates = {
            2016: (SEP, 19),
            2017: (SEP, 19),
            2018: (SEP, 19),
            2019: (SEP, 20),
            2020: (SEP, 19),
            2021: (SEP, 19),
            2022: (SEP, 19),
            2023: (SEP, 20),
            2024: (SEP, 19),
            2025: (SEP, 19),
            2026: (SEP, 19),
            2027: (SEP, 19),
            2028: (SEP, 18),
            2029: (SEP, 19),
            2030: (SEP, 19),
            2031: (SEP, 19),
            2032: (SEP, 19),
        }
        if dt := constitution_day_dates.get(self._year):
            # Constitution Day.
            self._add_holiday(tr("संविधान दिवस"), dt)

        # Christmas Day.
        self._add_christmas_day(tr("क्रिसमसको दिन"))

        # Hindu holidays.

        # Maghe Sankranti.
        self._add_makar_sankranti(tr("माघे संक्रान्ति"))

        # Sonam Lhochhar.
        self._add_sonam_losar(tr("सोनम ल्होछार"))

        # Maha Shivaratri.
        self._add_maha_shivaratri(tr("महाशिवरात्रि"))

        # Gyalpo Lhosar.
        self._add_gyalpo_losar(tr("ग्याल्पो ल्होसार"))

        # Fagu Poornima.
        self._add_nepal_holi(tr("फागुपूर्णिमा"))

        # Fagu Poornima (Terai).
        self._add_holi(tr("फागुपूर्णिमा (तराई)"))

        # Buddha Jayanti.
        self._add_buddha_purnima(tr("बुद्ध जयन्ती"))

        # Fulpati.
        self._add_maha_saptami(tr("फुलपाती"))

        # Maha Ashtami.
        self._add_maha_ashtami(tr("महा अष्टमी"))

        # Maha Navami.
        self._add_maha_navami(tr("महा नवमी"))

        # Bijaya Dashami.
        self._add_dussehra(tr("विजया दशमी"))

        # Ekadashi (Dashain).
        self._add_papankusha_ekadashi(tr("एकादशी (दशैं)"))

        # Laxmi Pooja.
        self._add_diwali_india(tr("लक्ष्मीपूजा"))

        # Gai Tihar.
        self._add_gau_krida(tr("गाई तिहार"))

        # Gobardhan Pooja.
        self._add_govardhan_puja(tr("गोवर्धन पूजा"))

        # Mha Pooja.
        self._add_govardhan_puja(tr("म्ह पूजा"))

        # Bhai Tika.
        self._add_bhai_dooj(tr("भाइटीका"))

        # Chhath Parva.
        self._add_chhath_puja(tr("छठ पर्व"))

        # Tamu Lhochhar.
        self._add_tamu_losar(tr("तमु ल्हाेसार"))

        # Islamic holidays.

        # Eid al-Fitr.
        self._add_eid_al_fitr_day(tr("ईद (ईद उल फित्र)"))

        # Eid al-Adha.
        self._add_eid_al_adha_day(tr("बकर ईद (ईद उल अजहा)"))

        # Removed by MoHA between 2019-2020.
        self._add_non_continuous_holidays()

    def _populate_workday_holidays(self):
        self._add_non_continuous_holidays(is_workday=True)


class NP(Nepal):
    pass


class NPL(Nepal):
    pass


class NepalHinduHolidays(_CustomHinduHolidays):
    # Maghe Sankranti.
    MAKAR_SANKRANTI_DATES = {
        2022: (JAN, 15),
        2023: (JAN, 15),
        2024: (JAN, 15),
        2026: (JAN, 15),
    }

    # Fagu Poornima (Terai).
    HOLI_DATES = {
        2023: (MAR, 7),
        2026: (MAR, 3),
    }

    # Ram Navami.
    RAM_NAVAMI_DATES = {
        2026: (MAR, 27),
    }

    # Nepal New Year.
    VAISAKHI_DATES = {
        2025: (APR, 14),
    }

    # Janai Punima.
    RAKSHA_BANDHAN_DATES = {
        2022: (AUG, 12),
        2023: (AUG, 31),
    }

    # Shree Krishna Janmashtami.
    JANMASHTAMI_DATES = {
        2023: (SEP, 6),
    }

    # Maha Ashtami.
    MAHA_ASHTAMI_DATES = {
        2020: (OCT, 24),
    }

    # Maha Navami.
    MAHA_NAVAMI_DATES = {
        2020: (OCT, 25),
    }

    # Bijaya Dashami.
    DUSSEHRA_DATES = {
        2020: (OCT, 26),
    }

    # Gobardhan Pooja.
    GOVARDHAN_PUJA_DATES = {
        2020: (NOV, 16),
        2022: (OCT, 26),
        2023: (NOV, 14),
    }

    # Chhath Parva.
    CHHATH_PUJA_DATES = {
        2025: (OCT, 27),
    }


class NepalIslamicHolidays(_CustomIslamicHolidays):
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2010, 2025)
    EID_AL_ADHA_DATES = {
        2014: (OCT, 6),
        2015: (SEP, 25),
        2016: (SEP, 13),
    }

    EID_AL_FITR_DATES_CONFIRMED_YEARS = (2010, 2025)
    EID_AL_FITR_DATES = {
        2010: (SEP, 10),
        2013: (AUG, 8),
    }


class NepalStaticHolidays:
    """Nepal special holidays.

    References:
        * [Death of Krishna Prasad Bhattarai](https://web.archive.org/web/20250527164153/https://myrepublica.nagariknetwork.com/news/27618/)
        * [Death of Sushil Koirala](https://web.archive.org/web/20250527164210/https://kathmandupost.com/miscellaneous/2016/02/09/cabinet-decision-koirala-to-be-honoured-with-state-funeral)
        * [Crash of Yeti Airlines Flight 691](https://web.archive.org/web/20250123204057/https://edition.cnn.com/2023/01/15/asia/nepal-yeti-airlines-crash-intl-hnk/index.html)
        * [People's War Day Instituted](https://web.archive.org/web/20241211215602/https://kathmandupost.com/national/2023/02/12/government-announces-public-holiday-on-monday-to-mark-people-s-war-day)
        * [People's War Day Annulled](https://web.archive.org/web/20240111230228/https://kathmandupost.com/national/2023/12/29/supreme-court-annuls-public-holiday-on-people-s-war-day)
        * [Death of Subas Chandra Nembang](https://web.archive.org/web/20241214193305/https://en.nepalkhabar.com/news/detail/6023/)
    """

    # Day of National Mourning.
    name_day_of_national_mourning = tr("राष्ट्रिय शोक दिवस")

    # Tihar Holiday.
    name_tihar_holiday = tr("तिहार बिदा")

    special_public_holidays = {
        2011: (MAR, 6, name_day_of_national_mourning),
        2016: (FEB, 10, name_day_of_national_mourning),
        2021: (NOV, 7, name_tihar_holiday),
        2022: (OCT, 28, name_tihar_holiday),
        2023: (
            (JAN, 16, name_day_of_national_mourning),
            # People War's Day.
            (FEB, 13, tr("जनयुद्ध दिवस")),
            (SEP, 14, name_day_of_national_mourning),
            (NOV, 16, name_tihar_holiday),
        ),
        2024: (NOV, 4, name_tihar_holiday),
        2025: (OCT, 24, name_tihar_holiday),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from datetime import date
from gettext import gettext as tr

from holidays.calendars.gregorian import (
    MAR,
    SUN,
    _get_all_sundays,
    _get_nth_weekday_from,
    _timedelta,
)
from holidays.constants import BANK, DE_FACTO, OPTIONAL, PUBLIC
from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.holiday_base import HolidayBase


class Sweden(HolidayBase, ChristianHolidays, InternationalHolidays):
    """Sweden holidays.

    References:
        * <https://sv.wikipedia.org/wiki/Helgdagar_i_Sverige>
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Sweden>
        * [Act 1952:48](https://web.archive.org/web/20250518071409/https://blogs.loc.gov/law/2023/06/midsommar-becomes-weekend-holiday-in-sweden-in-1953-pic-of-the-week/)
        * [Act 1989:253](https://web.archive.org/web/20250414065223/https://www.riksdagen.se/sv/dokument-lagar/dokument/svensk-forfattningssamling/lag-1989253-om-allmanna-helgdagar_sfs-1989-253)
        * <https://sv.wikipedia.org/wiki/Första_maj>
        * <https://sv.wikipedia.org/wiki/Sveriges_nationaldag>
        * <https://sv.wikipedia.org/wiki/Midsommarafton>
        * [Bank Holidays 2025](https://web.archive.org/web/20250811112642/https://www.riksbank.se/sv/press-och-publicerat/kalender/helgdagar-2025/)
        * [Swedish Annual Leave Law (SFS 1977:480)](https://web.archive.org/web/20260106114757/https://www.riksdagen.se/sv/dokument-och-lagar/dokument/svensk-forfattningssamling/semesterlag-1977480_sfs-1977-480/)

    In Sweden, ALL sundays are considered a holiday.
    Initialize this class with `include_sundays=False` to not include sundays as a holiday.

    Supported holiday categories:

    - PUBLIC: Official public holidays with general time off
    - DE_FACTO: Holidays treated equivalently to public holidays by law
    - BANK: Banking institution holidays
    - OPTIONAL: Optional or cultural observances

    The DE_FACTO category includes:
        - Midsommarafton (Midsummer Eve)
        - Julafton (Christmas Eve)
        - Nyårsafton (New Year's Eve)

    According to Swedish Annual Leave Law (SFS 1977:480, Section 7): "Med söndag
    jämställs allmän helgdag samt midsommarafton, julafton och nyårsafton"
    (Sundays are equivalent to public holidays as well as Midsummer's Eve,
    Christmas Eve, and New Year's Eve).

    These holidays are not official public holidays but must be treated as
    non-working days. For accurate `is_working_day()` calculations, use:
        `Sweden(categories=(PUBLIC, DE_FACTO))`
    """

    # %s (from 2pm).
    begin_time_label = tr("%s (från kl. 14.00)")
    country = "SE"
    default_language = "sv"
    # Act 1952:48.
    start_year = 1953
    supported_categories = (BANK, DE_FACTO, OPTIONAL, PUBLIC)
    supported_languages = ("en_US", "sv", "th", "uk")

    def __init__(self, *args, include_sundays: bool = True, **kwargs):
        """
        Args:
            include_sundays:
                Whether to consider sundays as a holiday (which they are in Sweden)
        """
        self.include_sundays = include_sundays
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("Nyårsdagen"))

        # Epiphany.
        self._add_epiphany_day(tr("Trettondedag jul"))

        if self._year <= 1989:
            # According to Act 1952:48, Feast of the Annunciation is
            # "on the Sunday between March 22 and 28 or, if this Sunday is Palm Sunday
            # or Easter Sunday, on the Sunday before Palm Sunday".
            dt = _get_nth_weekday_from(1, SUN, date(self._year, MAR, 22))
            self._add_holiday(
                # Feast of the Annunciation.
                tr("Marie bebådelsedag"),
                _timedelta(self._easter_sunday, -14)
                if dt == _timedelta(self._easter_sunday, -7) or dt == self._easter_sunday
                else dt,
            )

        # Good Friday.
        self._add_good_friday(tr("Långfredagen"))

        # Easter Sunday.
        self._add_easter_sunday(tr("Påskdagen"))

        # Easter Monday.
        self._add_easter_monday(tr("Annandag påsk"))

        # May Day.
        self._add_labor_day(tr("Första maj"))

        # Ascension Day.
        self._add_ascension_thursday(tr("Kristi himmelsfärdsdag"))

        if self._year >= 2005:
            # National Day.
            self._add_holiday_jun_6(tr("Nationaldagen"))

        # Whit Sunday.
        self._add_whit_sunday(tr("Pingstdagen"))

        if self._year <= 2004:
            # Whit Monday.
            self._add_whit_monday(tr("Annandag pingst"))

        # Midsummer Day.
        self._add_holiday_1st_sat_from_jun_20(tr("Midsommardagen"))

        # All Saints' Day.
        self._add_holiday_1st_sat_from_oct_31(tr("Alla helgons dag"))

        # Christmas Day.
        self._add_christmas_day(tr("Juldagen"))

        # Second Day of Christmas.
        self._add_christmas_day_two(tr("Annandag jul"))

        # Optionally add all Sundays of the year.
        if self.include_sundays:
            for dt in _get_all_sundays(self._year):
                # Sunday.
                self._add_holiday(tr("Söndag"), dt)

    def _populate_de_facto_holidays(self):
        # Midsummer Eve.
        self._add_holiday_1st_fri_from_jun_19(tr("Midsommarafton"))

        # Christmas Eve.
        self._add_christmas_eve(tr("Julafton"))

        # New Year's Eve.
        self._add_new_years_eve(tr("Nyårsafton"))

    def _populate_common(self):
        """Populate holidays that are both optional and bank holidays."""

        # Twelfth Night.
        self._add_holiday_jan_5(self.tr(self.begin_time_label) % self.tr("Trettondagsafton"))

        # Maundy Thursday.
        self._add_holy_thursday(self.tr(self.begin_time_label) % self.tr("Skärtorsdagen"))

        # Walpurgis Night.
        self._add_holiday_apr_30(self.tr(self.begin_time_label) % self.tr("Valborgsmässoafton"))

        self._add_holiday_1st_fri_from_oct_30(
            # All Saints' Eve.
            self.tr(self.begin_time_label) % self.tr("Allahelgonsafton")
        )

    def _populate_bank_holidays(self):
        self._populate_common()
        self._populate_de_facto_holidays()

        self._add_holiday_38_days_past_easter(
            # Day before Ascension Day.
            self.tr(self.begin_time_label) % self.tr("Dag före Kristi himmelsfärdsdag")
        )

        self._add_holiday_1st_thu_from_jun_18(
            # Day before Midsummer Eve.
            self.tr(self.begin_time_label) % self.tr("Dag före Midsommarafton")
        )

        self._add_holiday_47_days_past_easter(
            # Day before Whitsun Eve.
            self.tr(self.begin_time_label) % self.tr("Dag före Pingstafton")
        )

        # Day before Christmas Eve.
        self._add_holiday_dec_23(self.tr(self.begin_time_label) % self.tr("Dag före Julafton"))

        # Day before New Year's Eve.
        self._add_holiday_dec_30(self.tr(self.begin_time_label) % self.tr("Dag före Nyårsafton"))

    def _populate_optional_holidays(self):
        self._populate_common()

        # Holy Saturday.
        self._add_holy_saturday(tr("Påskafton"))

        # Days between a holiday and a weekend are in Swedish called "klämdagar" (squeeze days).
        # There is one permanent "klämdag" every year - Friday after Ascension Day.

        # Squeeze day.
        self._add_holiday_40_days_past_easter(tr("Klämdag"))

        # Whitsun Eve.
        self._add_holiday_48_days_past_easter(tr("Pingstafton"))


class SE(Sweden):
    pass


class SWE(Sweden):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.holiday_base import HolidayBase


class BrasilBolsaBalcao(HolidayBase, ChristianHolidays, InternationalHolidays):
    """Brasil, Bolsa, Balcão holidays.

    References:
        * [Decreto n. 155-B, de 14.01.1890](https://web.archive.org/web/20241226133739/https://www2.camara.leg.br/legin/fed/decret/1824-1899/decreto-155-b-14-janeiro-1890-517534-publicacaooriginal-1-pe.html)
        * [Decreto n. 19.488, de 15.12.1930](https://web.archive.org/web/20241006041503/http://camara.leg.br/legin/fed/decret/1930-1939/decreto-19488-15-dezembro-1930-508040-republicacao-85201-pe.html)
        * [Decreto n. 22.647, de 17.04.1933](https://web.archive.org/web/20220414202918/https://www2.camara.leg.br/legin/fed/decret/1930-1939/decreto-22647-17-abril-1933-558774-publicacaooriginal-80337-pe.html)
        * [Lei n. 662, de 6.04.1949](https://web.archive.org/web/20240913060643/https://www2.camara.leg.br/legin/fed/lei/1940-1949/lei-662-6-abril-1949-347136-publicacaooriginal-1-pl.html)
        * [Lei n. 6.802, de 30.06.1980](https://web.archive.org/web/20250206215621/http://www.planalto.gov.br/ccivil_03/leis/L6802.htm)
        * [Resolução n. 2.516, de 29.06.1998](https://web.archive.org/web/20250428203302/https://www.bcb.gov.br/pre/normativos/res/1998/pdf/res_2516_v2_P.pdf)
        * [Lei n. 14.759, de 21.12.2023](https://web.archive.org/web/20250402234552/https://www2.camara.leg.br/legin/fed/lei/2023/lei-14759-21-dezembro-2023-795091-publicacaooriginal-170522-pl.html)

    Historical data:
        * [Feriados ANBIMA 2001-2099](https://web.archive.org/web/20241209033536/https://www.anbima.com.br/feriados/)
        * [Calendario de negociação B3](https://web.archive.org/web/20250126154858/https://www.b3.com.br/pt_br/solucoes/plataformas/puma-trading-system/para-participantes-e-traders/calendario-de-negociacao/feriados/)
    """

    market = "BVMF"
    default_language = "pt_BR"
    supported_languages = ("en_US", "pt_BR", "uk")
    # Decreto n. 155-B, de 14.01.1890
    # Curiously enough, 1890 is also the year of foundation of the
    # São Paulo Stock Exchange, which would later become the B3.
    start_year = 1890

    def __init__(self, *args, **kwargs) -> None:
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # Universal Fraternization Day.
        self._add_new_years_day(tr("Confraternização Universal"))

        # Carnival.
        carnival_name = tr("Carnaval")
        self._add_carnival_monday(carnival_name)
        self._add_carnival_tuesday(carnival_name)

        # Resolução n. 2.516, de 29.06.1998
        if self._year <= 1999:
            # Holy Thursday.
            self._add_holy_thursday(tr("Quinta-feira Santa"))

        # Good Friday.
        self._add_good_friday(tr("Sexta-feira Santa"))

        # Tiradentes' Day was revoked by president Getúlio Vargas in
        # december of 1930, through Decreto n. 19.488, but reinstated
        # by the same president in 1933, through Decreto n. 22.647.
        if self._year not in {1931, 1932}:
            # Tiradentes' Day.
            self._add_holiday_apr_21(tr("Tiradentes"))

        if self._year >= 1925:
            # Worker's Day.
            self._add_labor_day(tr("Dia do Trabalhador"))

        # Corpus Christi.
        self._add_corpus_christi_day(tr("Corpus Christi"))

        # Independence Day.
        self._add_holiday_sep_7(tr("Independência do Brasil"))

        # Lei n. 6.802, de 30.06.1980
        if self._year >= 1980:
            # Our Lady of Aparecida.
            self._add_holiday_oct_12(tr("Nossa Senhora Aparecida"))

        # All Souls' Day.
        self._add_all_souls_day(tr("Finados"))

        # Republic Proclamation Day.
        self._add_holiday_nov_15(tr("Proclamação da República"))

        # Lei n. 14.759, de 21.12.2023
        if self._year >= 2024:
            # National Day of Zumbi and Black Awareness.
            self._add_holiday_nov_20(tr("Dia Nacional de Zumbi e da Consciência Negra"))

        if self._year >= 1922:
            # Christmas Day.
            self._add_christmas_day(tr("Natal"))


class BVMF(BrasilBolsaBalcao):
    pass


class B3(BrasilBolsaBalcao):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.julian import JULIAN_CALENDAR
from holidays.groups import ChristianHolidays, InternationalHolidays, IslamicHolidays
from holidays.observed_holiday_base import ObservedHolidayBase, SAT_SUN_TO_NEXT_WORKDAY


class Kosovo(ObservedHolidayBase, ChristianHolidays, InternationalHolidays, IslamicHolidays):
    """Kosovo holidays.

    References:
        * <https://web.archive.org/web/20251105011859/https://bqk-kos.org/kalendari-i-festave/>
        * [Law No. 03/L-064](https://web.archive.org/web/20231127220229/https://gzk.rks-gov.net/ActDocumentDetail.aspx?ActID=2539)
    """

    country = "XK"
    default_language = "sq"
    # %s (estimated).
    estimated_label = tr("%s (e vlerësuar)")
    # %s (observed).
    observed_label = tr("%s (ditë pushimi e shtyrë)")
    # %s (observed, estimated).
    observed_estimated_label = tr("%s (ditë pushimi e shtyrë, e vlerësuar)")
    supported_languages = ("en_US", "sq", "sr")
    # First full year after independence (2008-02-17).
    start_year = 2009

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(self, show_estimated=islamic_show_estimated)
        kwargs.setdefault("observed_rule", SAT_SUN_TO_NEXT_WORKDAY)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        dts_observed = set()

        # New Year's Day.
        name = tr("Viti i Ri")
        dts_observed.add(self._add_new_years_day(name))
        dts_observed.add(self._add_new_years_day_two(name))

        # Orthodox Christmas Day.
        dts_observed.add(self._add_christmas_day(tr("Krishtlindjet Ortodokse"), JULIAN_CALENDAR))

        dts_observed.add(
            # Independence Day.
            self._add_holiday_feb_17(tr("Dita e Pavarësisë së Republikës së Kosovës"))
        )

        # Catholic Easter.
        dts_observed.add(self._add_easter_sunday(tr("Pashkët Katolike")))

        # Orthodox Easter.
        dts_observed.add(self._add_easter_sunday(tr("Pashkët Ortodokse"), JULIAN_CALENDAR))

        dts_observed.add(
            # Constitution Day.
            self._add_holiday_apr_9(tr("Dita e Kushtetutës së Republikës së Kosovës"))
        )

        # International Workers' Day.
        dts_observed.add(self._add_labor_day(tr("Dita Ndërkombëtare e Punës")))

        # Europe Day.
        dts_observed.add(self._add_europe_day(tr("Dita e Evropës")))

        # Catholic Christmas Day.
        dts_observed.add(self._add_christmas_day(tr("Krishtlindjet Katolike")))

        # Eid al-Fitr.
        dts_observed.update(self._add_eid_al_fitr_day(tr("Bajrami i Madh, dita e parë")))

        # Eid al-Adha.
        dts_observed.update(self._add_eid_al_adha_day(tr("Bajrami i Vogël, dita e parë")))

        if self.observed:
            self._populate_observed(dts_observed)


class XK(Kosovo):
    pass


class XKK(Kosovo):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import APR, MAY, JUN, JUL, SEP, DEC
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    MON_TO_NEXT_TUE,
    SAT_SUN_TO_NEXT_MON,
    SAT_SUN_TO_NEXT_MON_TUE,
)


class UnitedKingdom(ObservedHolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays):
    """United Kingdom holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_the_United_Kingdom>
        * <https://archive.org/details/treatiseonbanki00walk/page/334/mode/2up>
        * <https://web.archive.org/web/20250426173714/https://www.gov.uk/bank-holidays>
        * <https://web.archive.org/web/20250423223001/https://www.timeanddate.com/holidays/uk>

    The Anniversary of the Battle of the Boyne bank holiday is proclaimed annually by the
    Secretary of State for Northern Ireland.

    In-Lieu observance was first provided in the Bank Holidays Extension Act 1875.
    """

    country = "GB"
    default_language = "en_GB"
    # %s (observed).
    observed_label = tr("%s (observed)")
    subdivisions: tuple[()] | tuple[str, ...] = (
        "ENG",  # England
        "NIR",  # Northern Ireland
        "SCT",  # Scotland
        "WLS",  # Wales
    )
    subdivisions_aliases = {
        "England": "ENG",
        "Northern Ireland": "NIR",
        "Scotland": "SCT",
        "Wales": "WLS",
    }
    supported_languages = ("en_GB", "en_US", "th")
    # Bank Holidays Act 1871.
    start_year = 1872
    _deprecated_subdivisions = ("UK",)

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, UnitedKingdomStaticHolidays)
        kwargs.setdefault("observed_rule", SAT_SUN_TO_NEXT_MON)
        # Bank Holidays Extension Act 1875.
        kwargs.setdefault("observed_since", 1875)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self) -> None:
        # Good Friday.
        self._add_good_friday(tr("Good Friday"))

        if self._year >= 1978:
            # May Day.
            name = tr("May Day")
            if self._year in {1995, 2020}:
                self._add_holiday_may_8(name)
            else:
                self._add_holiday_1st_mon_of_may(name)

        if self._year >= 1971:
            spring_bank_dates = {
                2002: (JUN, 4),
                2012: (JUN, 4),
                2022: (JUN, 2),
            }
            # Spring Bank Holiday.
            name = tr("Spring Bank Holiday")
            if dt := spring_bank_dates.get(self._year):
                self._add_holiday(name, dt)
            else:
                self._add_holiday_last_mon_of_may(name)

    def _populate_common(self):
        # Easter Monday.
        self._add_easter_monday(tr("Easter Monday"))

        if self._year >= 1971:
            # Late Summer Bank Holiday.
            self._add_holiday_last_mon_of_aug(tr("Late Summer Bank Holiday"))
        else:
            # Whit Monday.
            self._add_whit_monday(tr("Whit Monday"))

    def _populate_subdiv_holidays(self):
        if self.subdiv != "SCT":
            if self._year >= 1975:
                # New Year's Day.
                self._add_observed(self._add_new_years_day(tr("New Year's Day")))

            self._add_observed(
                # Christmas Day.
                self._add_christmas_day(tr("Christmas Day")),
                rule=SAT_SUN_TO_NEXT_MON_TUE,
            )

            self._add_observed(
                # Boxing Day.
                self._add_christmas_day_two(tr("Boxing Day")),
                rule=SAT_SUN_TO_NEXT_MON_TUE,
            )

        super()._populate_subdiv_holidays()

    def _populate_subdiv_eng_public_holidays(self):
        self._populate_common()

    def _populate_subdiv_nir_public_holidays(self):
        self._populate_common()

        if self._year >= 1903:
            # Saint Patrick's Day.
            self._add_observed(self._add_saint_patricks_day(tr("Saint Patrick's Day")))

        # Battle of the Boyne.
        self._add_observed(self._add_holiday_jul_12(tr("Battle of the Boyne")))

    def _populate_subdiv_sct_public_holidays(self):
        # New Year's Day.
        jan_1 = self._add_new_years_day(tr("New Year's Day"))

        self._add_observed(
            # New Year Holiday.
            self._add_new_years_day_two(tr("New Year Holiday")),
            rule=SAT_SUN_TO_NEXT_MON_TUE + MON_TO_NEXT_TUE,
        )
        self._add_observed(jan_1)

        # Summer Bank Holiday.
        self._add_holiday_1st_mon_of_aug(tr("Summer Bank Holiday"))

        if self._year >= 2006:
            # Saint Andrew's Day.
            self._add_observed(self._add_holiday_nov_30(tr("Saint Andrew's Day")))

        self._add_observed(
            # Christmas Day.
            self._add_christmas_day(tr("Christmas Day")),
            rule=SAT_SUN_TO_NEXT_MON_TUE if self._year >= 1974 else SAT_SUN_TO_NEXT_MON,
        )

        if self._year >= 1974:
            self._add_observed(
                # Boxing Day.
                self._add_christmas_day_two(tr("Boxing Day")),
                rule=SAT_SUN_TO_NEXT_MON_TUE,
            )

    def _populate_subdiv_wls_public_holidays(self):
        self._populate_common()


class UK(UnitedKingdom):
    pass


class GB(UnitedKingdom):
    pass


class GBR(UnitedKingdom):
    pass


class UnitedKingdomStaticHolidays:
    """United Kingdom special holidays.

    References:
        <https://web.archive.org/web/20260214143756/https://privycouncil.independent.gov.uk/wp-content/uploads/2026/02/Scotland-Bank-Holiday-Proclamation.pdf>
    """

    special_public_holidays = {
        # Silver Jubilee of Elizabeth II.
        1977: (JUN, 7, tr("Silver Jubilee of Elizabeth II")),
        # Wedding of Charles and Diana.
        1981: (JUL, 29, tr("Wedding of Charles and Diana")),
        # Millennium Celebrations.
        1999: (DEC, 31, tr("Millennium Celebrations")),
        # Golden Jubilee of Elizabeth II.
        2002: (JUN, 3, tr("Golden Jubilee of Elizabeth II")),
        # Wedding of William and Catherine.
        2011: (APR, 29, tr("Wedding of William and Catherine")),
        # Diamond Jubilee of Elizabeth II.
        2012: (JUN, 5, tr("Diamond Jubilee of Elizabeth II")),
        2022: (
            # Platinum Jubilee of Elizabeth II.
            (JUN, 3, tr("Platinum Jubilee of Elizabeth II")),
            # State Funeral of Queen Elizabeth II.
            (SEP, 19, tr("State Funeral of Queen Elizabeth II")),
        ),
        # Coronation of Charles III.
        2023: (MAY, 8, tr("Coronation of Charles III")),
    }

    special_sct_public_holidays = {
        # Scotland's participation in the FIFA World Cup final.
        2026: (JUN, 15, tr("Scotland's participation in the FIFA World Cup final"))
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from datetime import date

from holidays.calendars.burmese import _BurmeseLunisolar
from holidays.calendars.gregorian import _timedelta


class BurmeseCalendarHolidays:
    """Burmese lunisolar calendar holidays."""

    def __init__(self) -> None:
        self._burmese_calendar = _BurmeseLunisolar()

    def _add_burmese_calendar_holiday(
        self, name: str, dt: date | None = None, days_delta: int = 0
    ) -> date | None:
        """Add Burmese calendar holiday."""

        if dt is None:
            return None

        if days_delta:
            dt = _timedelta(dt, days_delta)

        return self._add_holiday(name, dt)

    def _add_karen_new_year(self, name: str) -> set[date]:
        """Add Karen New Year holiday.

        The Karen New Year, also known as the Kayin New Year, is one of the major holidays
        celebrated by the Karen people. The Karen New Year falls on the first day of Pyatho,
        the tenth month in the Burmese calendar.
        https://en.wikipedia.org/wiki/Karen_New_Year

        Args:
            name:
                Holiday name.

        Returns:
            Set of dates of added holiday, empty if there is no holiday date for the current year.
        """
        return {
            dt
            for y in (self._year - 1, self._year)
            if (
                dt := self._add_burmese_calendar_holiday(
                    name, self._burmese_calendar.pyatho_waxing_moon_date(y)
                )
            )
        }

    def _add_kason_full_moon_day(self, name: str) -> date | None:
        """Add Full Moon Day of Kason holiday.

        Vesak is known as the Full Moon Day of Kason, which is the second month
        in the traditional Burmese calendar.
        https://en.wikipedia.org/wiki/Vesak#In_Myanmar_(Burma)

        Args:
            name:
                Holiday name.

        Returns:
            Date of added holiday, `None` if there is no holiday date for the current year.
        """
        return self._add_burmese_calendar_holiday(
            name, self._burmese_calendar.kason_full_moon_date(self._year)
        )

    def _add_myanmar_diwali(self, name: str) -> date | None:
        """Add Myanmar Diwali holiday.

        Diwali (Deepavali, Festival of Lights) is one of the most important festivals
        in Indian religions. In Myanmar, it is celebrated on first day of Tazaungmon,
        the eighth month of the Burmese calendar.
        https://en.wikipedia.org/wiki/Diwali

        Args:
            name:
                Holiday name.

        Returns:
            Date of added holiday, `None` if there is no holiday date for the current year.
        """
        return self._add_burmese_calendar_holiday(
            name, self._burmese_calendar.tazaungmon_waxing_moon_date(self._year)
        )

    def _add_myanmar_national_day(self, name: str) -> date | None:
        """Add Myanmar National Day holiday.

        National Day is a public holiday in Myanmar, marking the anniversary of the first
        university student strike at Rangoon University in 1920. The date is based on
        the traditional Burmese calendar, occurring on the 10th day following
        the full moon of Tazaungmon.
        https://en.wikipedia.org/wiki/National_Day_(Myanmar)

        Args:
            name:
                Holiday name.

        Returns:
            Date of added holiday, `None` if there is no holiday date for the current year.
        """
        return self._add_burmese_calendar_holiday(
            name, self._burmese_calendar.tazaungmon_full_moon_date(self._year), days_delta=+10
        )

    def _add_myanmar_new_year(
        self,
        name: str,
        extra_days_before: int = 0,
        extra_days_after: int = 0,
    ) -> set[date]:
        """Add Myanmar New Year (Thingyan, Water Festival).

        Thingyan, also known as the Myanmar New Year, is a festival that usually occurs
        in middle of April. Thingyan marks the transition from the old year to the new one,
        based on the traditional Myanmar lunisolar calendar. The festival usually spans
        four to five days.

        https://en.wikipedia.org/wiki/Thingyan.

        Args:
            name:
                Holiday name.

            extra_days_before:
                Number of additional holiday days preceding Akya.

            extra_days_after:
                Whether to add additional holiday days following Atat.

        Returns:
            Set of dates of added holiday, empty if there is no holiday date for the current year.
        """
        akya, atat = self._burmese_calendar.thingyan_dates(self._year)
        if akya is None or atat is None:
            return set()

        # Default length is from Thingyan Akyo (akya - 1) to New Year Day (atat + 1).
        # Optional additional days:
        # pre-Akya days (1 day in 2007-2016, 4 days in 2022-2023),
        # post-Atat days (3 or 4 days).
        begin = -1 - extra_days_before
        end = extra_days_after or (atat - akya).days + 1

        return {
            dt
            for delta in range(begin, end)
            if (dt := self._add_holiday(name, _timedelta(akya, delta)))
        }

    def _add_tabaung_full_moon_day(self, name: str) -> date | None:
        """Add Full Moon Day of Tabaung holiday.

        Māgha Pūjā (also written as Makha Bucha Day, Meak Bochea) is a Buddhist festival
        celebrated on the full moon day of Tabaung in Myanmar.
        https://en.wikipedia.org/wiki/Māgha_Pūjā#Myanmar_(Burma)

        Args:
            name:
                Holiday name.

        Returns:
            Date of added holiday, `None` if there is no holiday date for the current year.
        """
        return self._add_burmese_calendar_holiday(
            name, self._burmese_calendar.tabaung_full_moon_date(self._year - 1)
        )

    def _add_tazaungmon_full_moon_day(self, name: str) -> date | None:
        """Add Full Moon Day of Tazaungmon holiday.

        The Tazaungdaing Festival, also known as the Festival of Lights, held on the full moon day
        of Tazaungmon, the eighth month of the Burmese calendar.
        https://en.wikipedia.org/wiki/Tazaungdaing_festival

        Args:
            name:
                Holiday name.

        Returns:
            Date of added holiday, `None` if there is no holiday date for the current year.
        """
        return self._add_burmese_calendar_holiday(
            name, self._burmese_calendar.tazaungmon_full_moon_date(self._year)
        )

    def _add_thadingyut_full_moon_eve(self, name: str) -> date | None:
        """Add Pre-Full Moon Day of Thadingyut holiday.

        The Thadingyut Festival, also known as the Lighting Festival of Myanmar, is held
        on the full moon day of the Burmese lunar month of Thadingyut.
        https://en.wikipedia.org/wiki/Thadingyut_Festival

        Args:
            name:
                Holiday name.

        Returns:
            Date of added holiday, `None` if there is no holiday date for the current year.
        """
        return self._add_burmese_calendar_holiday(
            name, self._burmese_calendar.thadingyut_full_moon_date(self._year), days_delta=-1
        )

    def _add_thadingyut_full_moon_day(self, name: str) -> date | None:
        """Add Full Moon Day of Thadingyut holiday.

        The Thadingyut Festival, also known as the Lighting Festival of Myanmar, is held
        on the full moon day of the Burmese lunar month of Thadingyut.
        https://en.wikipedia.org/wiki/Thadingyut_Festival

        Args:
            name:
                Holiday name.

        Returns:
            Date of added holiday, `None` if there is no holiday date for the current year.
        """
        return self._add_burmese_calendar_holiday(
            name, self._burmese_calendar.thadingyut_full_moon_date(self._year)
        )

    def _add_thadingyut_full_moon_day_two(self, name: str) -> date | None:
        """Add Post-Full Moon Day of Thadingyut holiday.

        The Thadingyut Festival, also known as the Lighting Festival of Myanmar, is held
        on the full moon day of the Burmese lunar month of Thadingyut.
        https://en.wikipedia.org/wiki/Thadingyut_Festival

        Args:
            name:
                Holiday name.

        Returns:
            Date of added holiday, `None` if there is no holiday date for the current year.
        """
        return self._add_burmese_calendar_holiday(
            name, self._burmese_calendar.thadingyut_full_moon_date(self._year), days_delta=+1
        )

    def _add_waso_full_moon_day(self, name: str) -> date | None:
        """Add Full Moon Day of Waso holiday.

        Vassa is the three-month annual retreat observed by Theravada Buddhists.
        Vassa lasts for three lunar months, from the Burmese month of Waso to
        the Burmese month of Thadingyut.
        https://en.wikipedia.org/wiki/Vassa

        Args:
            name:
                Holiday name.

        Returns:
            Date of added holiday, `None` if there is no holiday date for the current year.
        """
        return self._add_burmese_calendar_holiday(
            name, self._burmese_calendar.waso_full_moon_date(self._year)
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import JAN
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.observed_holiday_base import ObservedHolidayBase, SUN_TO_NEXT_MON


class Monaco(ObservedHolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays):
    """Monaco holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Monaco>
        * <https://web.archive.org/web/20231212161716/https://en.service-public-entreprises.gouv.mc/Employment-and-social-affairs/Employment-regulations/Leave/Public-Holidays>
    """

    country = "MC"
    default_language = "fr"
    # %s (observed).
    observed_label = tr("%s (observé)")
    supported_languages = ("en_US", "fr", "uk")

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, MonacoStaticHolidays)
        kwargs.setdefault("observed_rule", SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_observed(self._add_new_years_day(tr("Le jour de l'An")))

        # Saint Devote's Day.
        self._add_holiday_jan_27(tr("La Sainte Dévote"))

        # Easter Monday.
        self._add_easter_monday(tr("Le lundi de Pâques"))

        # Labor Day.
        self._add_observed(self._add_labor_day(tr("Fête de la Travaille")))

        # Ascension Day.
        self._add_ascension_thursday(tr("L'Ascension"))

        # Whit Monday.
        self._add_whit_monday(tr("Le lundi de Pentecôte"))

        # Corpus Christi.
        self._add_corpus_christi_day(tr("La Fête Dieu"))

        # Assumption Day.
        self._add_observed(self._add_assumption_of_mary_day(tr("L'Assomption de Marie")))

        # All Saints' Day.
        self._add_observed(self._add_all_saints_day(tr("La Toussaint")))

        # Prince's Day.
        self._add_observed(self._add_holiday_nov_19(tr("La Fête du Prince")))

        # Immaculate Conception.
        dec_8 = self._add_immaculate_conception_day(tr("L'Immaculée Conception"))
        if self._year >= 2019:
            self._move_holiday(dec_8, show_observed_label=False)

        # Christmas Day.
        self._add_observed(self._add_christmas_day(tr("Noël")))


class MC(Monaco):
    pass


class MCO(Monaco):
    pass


class MonacoStaticHolidays:
    special_public_holidays = {
        # Public holiday.
        2015: (JAN, 7, tr("Jour férié")),
    }
